# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def processEvents():
    """ processEvents()
    
    Processes all GUI events (and thereby all visvis events).
    Users can periodically call this function during running
    an algorithm to keep the figures responsove.
    
    Note that IEP and IPython can integrate the GUI event loop to
    periodically update the GUI events when idle.
    
    Also see Figure.DrawNow()
    
    """
    
    app = vv.backends.currentBackend.app
    if app:
        app.ProcessEvents()

if __name__ == '__main__':
    import time
    l = vv.plot([1,2,3,1,4])
    for i in range(20):
        l.SetYdata([1+i/10.0, 2,3,1,4])
        vv.processEvents() # Process gui events
        time.sleep(0.1)



#!/usr/bin/env python
import visvis as vv

# Create figure and make it wider than the default
fig = vv.figure()
fig.position.w = 700

# Create first axes
a1 = vv.subplot(121)

# Display an image
im = vv.imread('astronaut.png') # returns a numpy array
texture2d = vv.imshow(im)
texture2d.interpolate = True # if False the pixels are visible when zooming in

# Display two lines (values obtained via vv.ginput())
x = [182, 180, 161, 153, 191, 237, 293, 300, 272, 267, 254]
y = [145, 131, 112, 59, 29, 14, 48, 91, 136, 137, 172]
line1 = vv.plot(x, y, ms='.', mw=4, lw=2)
#
x = [507, 498, 483, 438, 364, 299, 278, 280]
y = [483, 452, 389, 349, 347, 393, 448, 508]
line2 = vv.plot(x, y, ms='s', mw=4, lw=2)

# The appearance of the line objects can be set in their
# constructor, or by using their properties
line1.lc, line1.mc = 'g', 'b'
line2.lc, line2.mc = 'y', 'r'

# Display a legend
a1.legend = "Astronaut's face", "Astronaut's helmet"


# Create second axes (with a black background)
a2 = vv.subplot(122)
a2.bgcolor = 'k'
a2.axis.axisColor = 'w'

# Display a texture
vol = vv.aVolume(2) # returns a test volume as a numpy array
texture3d = vv.volshow(vol)

# Display a mesh using one of the "solid" functions
mesh = vv.solidTeapot((32,32,80), scaling=(50,50,50))
mesh.faceColor = 0.4, 1, 0.4
mesh.specular = 'r'

# Set orthographic projection
a2.camera.fov = 45

# Create labels for the axis
a2.axis.xLabel = 'x-axis'
a2.axis.yLabel = 'y-axis'
a2.axis.zLabel = 'z-axis'

# Enter main loop
app = vv.use() # let visvis chose a backend for me
app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2017, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module sliders

Implements a few slider widgets.

"""

import OpenGL.GL as gl

from visvis.utils.pypoints import Pointset, Point
from visvis.core.events import BaseEvent
from visvis.core.misc import PropWithDraw, Range, unichr, getColor
from visvis.core.axises import GetTicks
from visvis import Box, Label


class BaseSlider(Box):
    """ BaseSlider(parent)
    
    Abstract slider class forming the base for the
    Slider and RangeSlider classes.
    
    """
    
    def __init__(self, parent):
        Box.__init__(self, parent)
        
        # init size
        self.position.w = 300
        self.position.h = 40
        
        # Slider specific stuff
        self._fullRange = Range(0,1)
        self._range = Range(0,1)
        self._refRange = Range(0,1) # For sliding
        self._showTicks = False
        
        # Set bgcolor and edge
        self.bgcolor = 0.6, 0.8, 0.6
        self._textColor = 0, 0, 0
        self.edgeWidth = 1
        
        # Create label centered at the box
        self._label = Label(self)
        self._label.position = 0,0,1,1
        self._label.halign = 0
        self._label.bgcolor = None
        
        # State variables
        self._isOver = False
        self._sliderDown = False
        self._sliderRefx = 0.0
        
        # Pool of labels for tickmarks, to reuse them
        self._wobjects = [] # So we can have Text objects
        self._labelPool = {}
        
        # Calculate dots now
        self._SliderCalcDots()
        
        # Create new events
        self._eventSliding = BaseEvent(self)
        self._eventSliderChanged = BaseEvent(self)
        
        # To changes appearance on mouse over
        self.eventEnter.Bind(self._SliderOnEnter)
        self.eventLeave.Bind(self._SliderOnLeave)
        
        # Bind to events
        self.eventMouseDown.Bind(self._SliderOnDown)
        self.eventMouseUp.Bind(self._SliderOnUp)
        self.eventMotion.Bind(self._SliderOnMotion)
        self.eventPosition.Bind(self._SliderCalcDots)
    
    
    @property
    def eventSliding(self):
        """ Event fired when the user is sliding the slider. This can
        occur manu times during one sliding-opertaion.
        """
        return self._eventSliding
    
    
    @property
    def eventSliderChanged(self):
        """ Event fired when the user releases the moude while changing
        the slider.
        """
        return self._eventSliderChanged
    
    
    @PropWithDraw
    def fullRange():
        """ The full possible range for this slider.
        """
        def fget(self):
            return self._fullRange
        def fset(self, value):
            self._fullRange = Range(value)
            self._limitRangeAndSetText()
        return locals()
    
    @PropWithDraw
    def showTicks():
        """ Whether to show tickmarks (default False).
        """
        def fget(self):
            return self._showTicks
        def fset(self, value):
            self._showTicks = bool(value)
        return locals()
    
    @PropWithDraw
    def textColor():
        """ The color of the text (the ticks and value label).
        """
        def fget(self):
            return self._textColor
        def fset(self, value):
            value = getColor(value, 'setting textColor')
            if value != self._textColor:
                self._textColor = value
                # Update existing
                self._label.textColor = value
                for label in self._labelPool.values():
                    label.textColor = value
                self.Draw()
        return locals()
    
    
    def _SliderOnEnter(self, event):
        self._isOver = True
        self.Draw()
    
    
    def _SliderOnLeave(self, event):
        self._isOver = False
        self.Draw()
    
    
    def _SliderOnDown(self, event):
        
        # Calc positions in normalized units
        x0 = self._getNormalizedCurrentPos(event)
        x1, x2 = self._getNormalizedSliderLimits()
        
        # Determine offset
        w,h = self.position.size
        offset = 8.0 / max(w,h)
        
        if x0 < x1+offset:
            # Move on left edge
            self._sliderDown = 1
            self._sliderRefx = x1 - x0
        elif x0 > x2-offset:
            # Move on right edge
            self._sliderDown = 3
            self._sliderRefx = x2 - x0
        else:
            self._sliderDown = 2 # Move middle
            self._sliderRefx = (x1+x2)/2.0 - x0
        
        # Store original
        self._refRange = Range(self._range)
        
        # Update
        self.Draw()
    
    
    def _SliderOnMotion(self, event):
        
        if not self._sliderDown:
            return
        
        # Calc current position in normalized units
        x0 = self._getNormalizedCurrentPos(event)
        
        # Reset before updating
        self._range = Range(self._refRange)
        
        # New normalized position
        dx =  x0 + self._sliderRefx
        offset = self._fullRange.min
        
        if self._sliderDown == 1:
            # Move on left edge
            self._range.min = self._fullRange.range * dx + offset
        elif self._sliderDown == 3:
            # Move on right edge
            self._range.max = self._fullRange.range * dx + offset
        elif self._sliderDown == 2:
            # Move whole slider
            ra2 = self._refRange.range / 2.0
            mi = self._fullRange.range * dx - ra2 + offset
            ma = self._fullRange.range * dx + ra2 + offset
            self._range.Set(mi,ma)
        
        # Limit and update
        self._limitRangeAndSetText()
        self._eventSliding.Set()
        self._eventSliding.Fire()
        self.Draw()
    
    def _SliderOnUp(self, event):
        
        # Update values
        self._SliderOnMotion(event)
        
        # Release
        self._sliderDown = 0
        
        # Update
        self._eventSliderChanged.Set()
        self._eventSliderChanged.Fire()
    
    
    def _limitRangeAndSetText(self):
        """ _limitRangeAndSetText()
        To limit the range of the slider and to set the slider text.
        Different slider implementation want to do this
        differently.
        """
        pass # Abstact method
    
    
    def _getNormalizedCurrentPos(self, event):
        """ _getNormalizedCurrentPos(event)
        Get the current mouse position as a normalized range unit
        (between 0 and 1, with the fullRange as a references).
        """
        w,h = self.position.size
        if w > h:
            return float(event.x) / self.position.width
        else:
            return float(event.y) / self.position.height
    
    
    def _getNormalizedSliderLimits(self):
        """ _getNormalizedSliderLimits()
        Get the current limits of the slider expressed in normalized
        units (between 0 and 1, with the fullRange as a references).
        """
        # Short names
        R1 = self._range
        R2 = self._fullRange
        #
        factor = R2.range
        x1 = (R1.min - R2.min) / factor
        x2 = x1 + R1.range / factor
        #
        return x1, x2
    
    
    def _GetBgcolorToDraw(self):
        """ Can be overloaded to indicate mouse over in buttons.
        """
        clr = list(self._bgcolor)
        if self._isOver:
            clr = [c+0.05 for c in clr]
        return clr
    
    
    def _format_number(self, v):
        """ Get the format in which to display the slider limits.
        """
        # Zero range ... meh
        if self._fullRange.range == 0:
            return '%1.4g' % v
        
        # Establish needed precesion, taking into account the possibility
        # of high limits, but a small range.
        s1 = abs(self._fullRange.min / self._fullRange.range)
        s2 = abs(self._fullRange.max / self._fullRange.range)
        significance = len(str(int(max(s1, s2)))) + 2
        
        # Apply format
        fmt = '%%1.%ig' % significance
        txt = fmt % v
        
        if significance == 3 and self._fullRange.range > 10:
            # Large numbers, fixed precision
            if self._fullRange.range >= 100000:
                pass
            elif self._fullRange.range >= 1000:
                fmt = '%1.0f'
            elif self._fullRange.range >= 100:
                fmt = '%1.1f'
            elif self._fullRange.range >= 10:
                fmt = '%1.2f'
            elif self._fullRange.range >= 1:
                fmt = '%1.3f'
            txt = fmt % v
        
        elif 'e' not in txt:
            # Add zeros to avoid jumpy feel
            actual_significance = len(txt.lstrip('+-0').replace('.', ''))
            if actual_significance < significance:
                if '.' not in txt:
                    txt += '.'
                txt += '0' * (significance - actual_significance)
        
        return txt
    
    
    def _SliderCalcDots(self, event=None):
        
        # Init dots
        dots1, dots2 = Pointset(2), Pointset(2)
        
        # Get width height
        w,h = self.position.size
        
        # Fill pointsets
        if w > h:
            i = 5
            while i < h-5:
                dots1.append(2,i); dots1.append(5,i)
                dots2.append(-2,i); dots2.append(-5,i)
                i += 3
        else:
            i = 5
            while i < w-5:
                dots1.append(i,2); dots1.append(i,5)
                dots2.append(i,-2); dots2.append(i,-5)
                i += 3
        
        self._dots1, self._dots2 = dots1, dots2
    
    
    def OnDraw(self):
        
        # Draw bg color and edges
        Box.OnDraw(self)
        
        # Margin
        d1 = 2
        d2 = d1+1
        
        # Get normalize limits
        t1, t2 = self._getNormalizedSliderLimits()
        
        # Get widget shape
        w, h = self.position.size
        
        # Calculate real dimensions of patch
        if w > h:
            x1, x2 = max(d2, t1*w), min(w-d1, t2*w)
            y1, y2 = d1, h-d2
            #
            dots1 = self._dots1 + Point(x1, 0)
            dots2 = self._dots2 + Point(x2, 0)
            #
            diff = abs(x1-x2)
            #
            self._label.textAngle = 0
        else:
            x1, x2 = d2, w-d1
            y1, y2 = max(d1, t1*h), min(h-d2, t2*h)
            #
            dots1 = self._dots1 + Point(0, y1)
            dots2 = self._dots2 + Point(0, y2)
            #
            diff = abs(y1-y2)
            #
            self._label.textAngle = -90
        
        # Draw slider bit
        clr = self._bgcolor
        #if max(clr[0], clr[1], clr[2]) < 0.7:
        if clr[0] + clr[1] + clr[2] < 1.5:
            gl.glColor(1, 1, 1, 0.25)
        else:
            gl.glColor(0, 0, 0, 0.25)
        #
        gl.glBegin(gl.GL_POLYGON)
        gl.glVertex2f(x1,y1)
        gl.glVertex2f(x1,y2)
        gl.glVertex2f(x2,y2)
        gl.glVertex2f(x2,y1)
        gl.glEnd()
        
        
        # Draw dots
        if True:
            
            # Prepare
            gl.glColor(0,0,0,1)
            gl.glPointSize(1)
            gl.glDisable(gl.GL_POINT_SMOOTH)
            
            # Draw
            gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
            if isinstance(self, RangeSlider) and diff>5:
                gl.glVertexPointerf(dots1.data)
                gl.glDrawArrays(gl.GL_POINTS, 0, len(dots1))
            if diff>5:
                gl.glVertexPointerf(dots2.data)
                gl.glDrawArrays(gl.GL_POINTS, 0, len(dots2))
            gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
        
        
        if self._showTicks:
            
            # Reset color to black
            gl.glColor(0,0,0,1)
            
            # Draw ticks
            if w>h:
                p0 = Point(0, h)
                p1 = Point(w, h)
                delta = Point(0,3)
                halign, valign = 0, 0
                xoffset, yoffset = -8, -2
            else:
                p0 = Point(w, h)
                p1 = Point(w, 0)
                delta = Point(3,0)
                halign, valign = -1, 0
                xoffset, yoffset = 5, -8
            
            # Get tickmarks
            ticks, ticksPos, ticksText = GetTicks(p0, p1, self._fullRange)
            
            newLabelPool = {}
            linePieces = Pointset(2)
            for tick, pos, text in zip(ticks, ticksPos, ticksText):
                pos2 = pos + delta
                
                # Add line piece
                linePieces.append(pos); linePieces.append(pos2)
                
                # Create or reuse label
                if tick in self._labelPool:
                    label = self._labelPool.pop(tick)
                else:
                    label = Label(self, ' '+text+' ')
                    label.bgcolor = ''
                    label.textColor = self._textColor
                
                # Position label and set text alignment
                newLabelPool[tick] = label
                label.halign, label.valign = halign, valign
                label.position.x = pos2.x + xoffset
                label.position.w = 16
                label.position.y = pos2.y + yoffset
            
            # Clean up label pool
            for label in list(self._labelPool.values()):
                label.Destroy()
            self._labelPool = newLabelPool
            
            # Draw line pieces
            gl.glLineWidth(1)
            gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
            gl.glVertexPointerf(linePieces.data)
            gl.glDrawArrays(gl.GL_LINES, 0, len(linePieces))
            gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
        

class Slider(BaseSlider):
    """ Slider(parent, fullRange=(0,1), value=0.5)
    
    A slider with which a scalar value can be interactively changed.
    The slider can be horizontal or vertical, depending on its
    width/height ratio.
    
    """
    
    def __init__(self, parent, fullRange=(0,1), value=0.5):
        BaseSlider.__init__(self, parent)
        
        self.fullRange = fullRange
        self.value = value
    
    @PropWithDraw
    def value():
        """ The current value for this slider.
        """
        def fget(self):
            return self._range.max
        def fset(self, value):
            self._range.max = float(value)
            self._limitRangeAndSetText()
            #
            self._eventSliderChanged.Set()
            self._eventSliderChanged.Fire()
        return locals()
    
    def _limitRangeAndSetText(self):
        # Limit
        self._range.max = min(self._range.max, self._fullRange.max)
        self._range.max = max(self._range.max, self._fullRange.min)
        self._range.min = self._fullRange.min
        
        # Set text
        self._label.text = self._format_number(self._range.max)


class RangeSlider(BaseSlider):
    """ RangeSlider(parent, fullRange=(0,1), range=(0.25,0.75))
    
    A slider with which two scalar values (representing a range)
    can be interactively changed. The slider can be horizontal or
    vertical, depending on its width/height ratio.
    
    """
    
    def __init__(self, parent, fullRange=(0,1), range=(0.25,0.75)):
        BaseSlider.__init__(self, parent)
        
        self.fullRange = fullRange
        self.range = range
    
    
    @PropWithDraw
    def range():
        """ The current range for this slider.
        """
        def fget(self):
            return self._range
        def fset(self, value):
            self._range = Range(value)
            self._limitRangeAndSetText()
            #
            self._eventSliderChanged.Set()
            self._eventSliderChanged.Fire()
        return locals()
    
    def _limitRangeAndSetText(self):
        # Limit
        self._range.max = min(self._range.max, self._fullRange.max)
        self._range.max = max(self._range.max, self._fullRange.min)
        #
        self._range.min = max(self._range.min, self._fullRange.min)
        self._range.min = min(self._range.min, self._fullRange.max)
        
        # Set text
        self._label.text = (self._format_number(self._range.min) +
                            '  ' + unichr(8211) + '  ' +
                            self._format_number(self._range.max))



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import numpy as np
import visvis as vv
from visvis.utils.pypoints import Point, Pointset
from visvis.wobjects.polygonalModeling import BaseMesh

def getSpanVectors(normal, c, d):
    """ getSpanVectors(normal, prevA, prevB) -> (a,b)
    
    Given a normal, return two orthogonal vectors which are both orthogonal
    to the normal. The vectors are calculated so they match as much as possible
    the previous vectors.
    
    """
    
    # Calculate a from previous b
    a1 = d.cross(normal)
    
    if a1.norm() < 0.001:
        # The normal and  d point in same or reverse direction
        # -> Calculate b from previous a
        b1 = c.cross(normal)
        a1 = b1.cross(normal)
    
    # Consider the opposite direction
    a2 = -1 * a1
    if c.distance(a1) > c.distance(a2):
        a1 = a2
    
    # Ok, calculate b
    b1 = a1.cross(normal)
        
#     # Consider the opposite (don't: this would make backfacing faces)
#     b2 = -1 * b1
#     if d.distance(b1) > d.distance(b2):
#         b1 = b2

    # Done
    return a1.normalize(), b1.normalize()


def getCircle(angles_cos, angles_sin, a, b):
    """ getCircle(angles_cos, angles_sin, a, b) -> circle_cords
    
    Creates a circle of points around the origin,
    the circle is spanned by the vectors a and b.
    
    """
    X = np.empty((len(angles_cos),3),dtype=np.float32)
    X[:,0] = angles_cos * a.x + angles_sin * b.x
    X[:,1] = angles_cos * a.y + angles_sin * b.y
    X[:,2] = angles_cos * a.z + angles_sin * b.z
    
    return Pointset(X)


def lineToMesh(pp, radius, vertex_num, values=None):
    """ lineToMesh(pp, radius, vertex_num, values=None)
    
    From a line, create a mesh that represents the line as a tube with
    given diameter. Returns a BaseMesh instance.
    
    Parameters
    ----------
    pp : 3D Pointset
        The points along the line. If the first and last point are the same,
        the mesh-line is closed.
    radius : scalar
        The radius of the tube that is created. Radius can also be a
        sequence of values (containing a radius for each point).
    vertex_num : int
        The number of vertices to create along the circumference of the tube.
    values : list or numpy array (optional)
        A value per point. Can be Nx1, Nx2, Nx3 or Nx4. A list of scalars
        can also be given. The values are propagated to the mesh vertices
        and supplied as input to the Mesh constructor. This allows for example
        to define the color for the tube.
    
    """
    
    # we need this quite a bit
    pi = np.pi
    
    # process radius
    if hasattr(radius, '__len__'):
        if len(radius) != len(pp):
            raise ValueError('Len of radii much match len of points.')
        else:
            radius = np.array(radius, dtype=np.float32)
    else:
        radius = radius*np.ones((len(pp),), dtype=np.float32)
    
    # calculate vertex points for 2D circle
    angles = np.arange(0, pi*2-0.0001, pi*2/vertex_num)
    angle_cos = np.cos(angles)
    angle_sin = np.sin(angles)
    vertex_num2 = len(angles) # just to be sure
    
    # calculate distance between two line pieces (for smooth cylinders)
    dists = pp[1:].distance(pp[:-1])
    bufdist = min( radius.max(), dists.min()/2.2)
    
    # check if line is closed
    lclosed = np.all(pp[0]==pp[-1])
    
    # calculate normal vectors on each line point
    normals = pp[:-1].subtract( pp[1:] ).copy()
    if lclosed:
        normals.append( pp[0]-pp[1] )
    else:
        normals.append( pp[-2]-pp[-1] )
    normals = -1 * normals.normalize()
    
    # create list to store vertices
    vertices = Pointset(3)
    surfaceNormals = Pointset(3)
    
    # And a list for the values
    if values is None:
        vvalues = None
    elif isinstance(values, list):
        if len(values) != len(pp):
            raise ValueError('There must be as many values as points.')
        vvalues = Pointset(1)
    elif isinstance(values, np.ndarray):
        if values.ndim != 2:
            raise ValueError('Values must be Nx1, Nx2, Nx3 or Nx4.')
        if values.shape[0] != len(pp):
            raise ValueError('There must be as many values as points.')
        vvalues = Pointset(values.shape[1])
    elif vv.utils.pypoints.is_Pointset(values):
        if values.ndim > 4:
            raise ValueError('Can specify one to four values per point.')
        if len(values) != len(pp):
            raise ValueError('There must be as many values as points.')
        vvalues = Pointset(values.ndim)
    else:
        raise ValueError('Invalid value for values.')
    
    # Number of triangelized cylinder elements added to plot the 3D line
    n_cylinders = 0
    
    # Init a and b
    a, b = Point(0,0,1), Point(0,1,0)
    
    # Calculate the 3D circle coordinates of the first circle/cylinder
    a,b = getSpanVectors(normals[0], a, b)
    circm = getCircle(angle_cos, angle_sin, a, b)
    
    # If not a closed line, add half sphere made with 5 cylinders at line start
    if not lclosed:
        for j in range(5,0,-1):
            # Translate the circle on it's position on the line
            r = (1-(j/5.0)**2)**0.5
            circmp = float(r*radius[0])*circm + (pp[0]-(j/5.0)*bufdist*normals[0])
            # Calc normals
            circmn = ( pp[0].subtract(circmp) ).normalize()
            # Store the vertex list
            vertices.extend( circmp )
            surfaceNormals.extend( -1*circmn )
            if vvalues is not None:
                for iv in range(vertex_num2):
                    vvalues.append(values[0])
            n_cylinders += 1
    
    # Loop through all line pieces
    for i in range(len(pp)-1):
        
        ## Create main cylinder between two line points
        # which consists of two connected circles.
        
        # get normal and point
        normal1 = normals[i]
        point1 = pp[i]
        
        # calculate the 3D circle coordinates
        a,b = getSpanVectors(normal1, a, b)
        circm = getCircle(angle_cos, angle_sin, a, b)
        
        # Translate the circle, and store
        circmp = float(radius[i])*circm + (point1+bufdist*normal1)
        vertices.extend( circmp )
        surfaceNormals.extend( circm )
        if vvalues is not None:
            for iv in range(vertex_num2):
                vvalues.append(values[i])
        n_cylinders += 1
        
        # calc second normal and line
        normal2 = normals[i+1]
        point2 = pp[i+1]
        
        # Translate the circle, and store
        circmp = float(radius[i+1])*circm + (point2-bufdist*normal1)
        vertices.extend( circmp )
        surfaceNormals.extend( circm )
        if vvalues is not None:
            for iv in range(vertex_num2):
                vvalues.append(values[i+1])
        n_cylinders += 1
        
        
        ## Create in between circle to smoothly connect line pieces
        
        if not lclosed and i == len(pp)-2:
            break
        
        # get normal and point
        normal12 = (normal1 + normal2).normalize()
        tmp = (point2+bufdist*normal2) + (point2-bufdist*normal1)
        point12 = 0.5858*point2 + 0.4142*(0.5*tmp)
        
        # Calculate the 3D circle coordinates
        a,b = getSpanVectors(normal12, a, b)
        circm = getCircle(angle_cos, angle_sin, a, b)
        
        # Translate the circle, and store
        circmp = float(radius[i+1])*circm + point12
        vertices.extend( circmp )
        surfaceNormals.extend( circm )
        if vvalues is not None:
            for iv in range(vertex_num2):
                vvalues.append( 0.5*(values[i]+values[i+1]) )
        n_cylinders += 1
    
    
    # If not a closed line, add half sphere made with 5 cylinders at line start
    # Otherwise add the starting circle to the line end.
    if not lclosed:
        for j in range(0,6):
            # Translate the circle on it's position on the line
            r = (1-(j/5.0)**2)**0.5
            circmp = float(r*radius[-1])*circm + (pp[-1]+(j/5.0)*bufdist*normals[-1])
            # Calc normals
            circmn = ( pp[-1].subtract(circmp) ).normalize()
            # Store the vertex list
            vertices.extend( circmp )
            surfaceNormals.extend( -1*circmn )
            if vvalues is not None:
                for iv in range(vertex_num2):
                    vvalues.append(values[-1])
            n_cylinders += 1
    else:
        # get normal and point
        normal1 = normals[-1]
        point1 = pp[-1]
        
        # calculate the 3D circle coordinates
        a,b = getSpanVectors(normal1, a, b)
        circm = getCircle(angle_cos, angle_sin, a, b)
        
        # Translate the circle, and store
        circmp = float(radius[0])*circm + (point1+bufdist*normal1)
        vertices.extend( circmp )
        surfaceNormals.extend( circm )
        if vvalues is not None:
            for iv in range(vertex_num2):
                vvalues.append(values[-1])
        n_cylinders += 1
    
    
    # almost done, determine quad faces ...
    
    # define single faces
    #firstFace = [0, 1, vertex_num+1, vertex_num]
    #lastFace = [vertex_num-1, 0, vertex_num, 2*vertex_num-1]
    firstFace = [vertex_num, vertex_num+1, 1, 0]
    lastFace = [2*vertex_num-1, vertex_num, 0, vertex_num-1]
    
    # define single round
    oneRound = []
    for i in range(vertex_num-1):
        oneRound.extend( [val+i for val in firstFace] )
    oneRound.extend(lastFace)
    oneRound = np.array(oneRound, dtype=np.uint32)
    
    # calculate face data
    parts = []
    for i in range(n_cylinders-1):
        parts.append(oneRound+i*vertex_num)
    faces = np.concatenate(parts)
    faces.shape = faces.shape[0]//4, 4
    
    # Done!
    return BaseMesh(vertices, faces, surfaceNormals, vvalues)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2017, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv


def isosurface(im, isovalue=None, step=1, useClassic=False, useValues=False):
    """ isosurface(vol, isovalue=None, step=1, useClassic=False, useValues=False)
    
    Uses scikit-image to calculate the isosurface for the given 3D image.
    Returns a vv.BaseMesh object.
    
    Parameters
    ----------
    vol : 3D numpy array
        The volume for which to calculate the isosurface.
    isovalue : float
        The value at which the surface should be created. If not given or None,
        the average of the min and max of vol is used.
    step : int
        The stepsize for stepping through the volume. Larger steps yield
        faster but coarser results. The result shall always be topologically
        correct though.
    useClassic : bool
        If True, uses the classic marching cubes by Lorensen (1987) is used.
        This algorithm has many ambiguities and is not guaranteed to produce
        a topologically correct result.
    useValues : bool
        If True, the returned BaseMesh object will also have a value for
        each vertex, which is related to the maximum value in a local region
        near the isosurface.
    
    """
    
    from visvis.utils.iso import isosurface as _isosurface
    
    return _isosurface(im, isovalue, step, useClassic, useValues)


if __name__ == '__main__':
    vv.mesh(isosurface(vv.volread('stent')))



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module textures

Defines the texture base class and the Texture2D and Texture3D
wobjects.

2D textures can be visualized without using GLSL. If GLSL is enabled, it
allows using clim, colormap and antialiasing (aa property).

3D textures are rendered using GLSL shader programs. The shader can be
selected using texture3D.renderStyle = 'ray', where 'ray' can be the
name of any of the available fragment shaders.


"""

import OpenGL.GL as gl

import numpy as np

from visvis.utils.pypoints import Pointset
#
from visvis import Range, Wobject, Colormapable
from visvis.core.misc import PropWithDraw, DrawAfter
from visvis.core.misc import Transform_Translate, Transform_Scale
from visvis.core import shaders
#
from visvis.core import TextureObject
from visvis.wobjects.motion import MotionMixin


# A correction for the clim. For a datatype of uint8, the fragents
# are mapped between 0 and 1 for 0 and 255 respectively. For int8
# the values are mapped between -1 and 1 for -127 and 128 respectively.
climCorrection = { 'uint8':2**8, 'int8':2**7, 'uint16':2**16, 'int16':2**15,
                   'uint32':2**32, 'int32':2**31, 'float32':1, 'float64':1,
                   'bool':2**8}


def minmax(data):
    """ minmax(data)
    
    Get the min and max of the data, ignoring inf and nan.
    
    """
    
    # Check for inf and nan
    M1 = np.isnan(data)
    M2 = np.isinf(data)
    
    # Select all 'normal' elements
    if np.any(M1) or np.any(M2):
        data2 = data[ ~(M1|M2) ]
    else:
        data2 = data
    
    # Return min and max
    return data2.min(), data2.max()


class TextureObjectToVisualize(TextureObject):
    """ TextureObjectToVisualize(ndim, data, interpolate=False)
    
    A texture object aimed towards visualization.
    This is what is actually used in Texture2D and Texture3D objects.
    It has no propererties, but some private attributes
    which are set by the real interface (the Texture*D objects).
    Basically, it handles the color limits.
    
    """
    
    def __init__(self, ndim, data, interpolate=False):
        TextureObject.__init__(self, ndim)
        
        # interpolate?
        self._interpolate = interpolate
        
        # the limits
        self._clim = Range(0,1)
        self._climCorrection = 1.0
        self._climRef = Range(0,1) # the "original" range
        
        # init clim and colormap
        self._climRef.Set(*minmax(data))
        self._clim = self._climRef.Copy()
    
    
    def _UploadTexture(self, data, *args):
        """ "Overloaded" method to upload texture data
        """
        
        # Set alignment to 1. It is 4 by default, but my data array has no
        # strides, so in order for the image not to be distorted, I set it
        # to 1. I assume graphics cards can still render in hardware. If
        # not, I would have to add one or two rows to my data instead.
        gl.glPixelStorei(gl.GL_UNPACK_ALIGNMENT,1)
        
        # init transferfunctions and set clim to full range
        self._ScaleBias_init(data.dtype.name)
        
        # create texture
        TextureObject._UploadTexture(self, data, *args)
        
        # set interpolation and extrapolation parameters
        tmp1 = gl.GL_NEAREST
        tmp2 = {False:gl.GL_NEAREST, True:gl.GL_LINEAR}[self._interpolate]
        gl.glTexParameteri(self._texType, gl.GL_TEXTURE_MIN_FILTER, tmp1)
        gl.glTexParameteri(self._texType, gl.GL_TEXTURE_MAG_FILTER, tmp2)
        gl.glTexParameteri(self._texType, gl.GL_TEXTURE_WRAP_S, gl.GL_CLAMP)
        gl.glTexParameteri(self._texType, gl.GL_TEXTURE_WRAP_T, gl.GL_CLAMP)
        
        # reset transfer
        self._ScaleBias_afterUpload()
        
        # Set clamping. When testing the raycasting, comment these lines!
        if self._ndim==3:
            gl.glTexParameteri(self._texType, gl.GL_TEXTURE_WRAP_S, gl.GL_CLAMP)
            gl.glTexParameteri(self._texType, gl.GL_TEXTURE_WRAP_T, gl.GL_CLAMP)
            gl.glTexParameteri(self._texType, gl.GL_TEXTURE_WRAP_R, gl.GL_CLAMP)
    
    
    def _UpdateTexture(self, data, *args):
        """ "Overloaded" method to update texture data
        """
        
        # init transferfunctions and set clim to full range
        self._ScaleBias_init(data.dtype.name)
        
        # create texture
        TextureObject._UpdateTexture(self, data, *args)
        
        # Update interpolation
        tmp = {False:gl.GL_NEAREST, True:gl.GL_LINEAR}[self._interpolate]
        gl.glTexParameteri(self._texType, gl.GL_TEXTURE_MAG_FILTER, tmp)
        
        # reset transfer
        self._ScaleBias_afterUpload()
    
    
    def _ScaleBias_init(self, datatype):
        """ Given the climRef (which is set to data.min() and data.max())
        in constructor, set the scale
        and bias for copying data to opengl memory. Correct for the dataype.
        Also set the default value for clim to the full data range.
        
        More info: OpenGL will map the full range of the datatype
        to 0:1 for unsigned datatypes, and to -1:1 for signed datatypes.
        For floats, 0:1 is mapped to 0:1. We modify the scale, such that
        the full range of the data (not the datatype) is scaled between 0:1.
        This way we can also visualize float data with values other than 0:1.
        """
        # store data range as a reference and init clim with that
        #self._clim = self._climRef.Copy()
        # calculate scale and bias
        ran = self._climRef.range
        if ran==0:
            ran = 1.0
        scale = climCorrection[datatype] / ran
        bias = -self._climRef.min / ran
        # set transfer functions
        gl.glPixelTransferf(gl.GL_RED_SCALE, scale)
        gl.glPixelTransferf(gl.GL_GREEN_SCALE, scale)
        gl.glPixelTransferf(gl.GL_BLUE_SCALE, scale)
        gl.glPixelTransferf(gl.GL_RED_BIAS, bias)
        gl.glPixelTransferf(gl.GL_GREEN_BIAS, bias)
        gl.glPixelTransferf(gl.GL_BLUE_BIAS, bias)
    
    
    def _ScaleBias_afterUpload(self):
        """ Reset the transferfunctions. """
        gl.glPixelTransferf(gl.GL_RED_SCALE, 1.0)
        gl.glPixelTransferf(gl.GL_GREEN_SCALE, 1.0)
        gl.glPixelTransferf(gl.GL_BLUE_SCALE, 1.0)
        gl.glPixelTransferf(gl.GL_RED_BIAS, 0.0)
        gl.glPixelTransferf(gl.GL_GREEN_BIAS, 0.0)
        gl.glPixelTransferf(gl.GL_BLUE_BIAS, 0.0)
    
    
    def _ScaleBias_get(self):
        """ Given clim, get scale and bias to apply in shader."""
        # ger ranges and correct if zero
        r1, r2 = self._clim.range, self._climRef.range
        if r1==0:
            r1 = 1.0
        if r2==0:
            r2 = 1.0
        # calculate scale and bias
        scale = self._climRef.range / r1
        bias = (self._climRef.min - self._clim.min) / r2
        return scale, bias


class BaseTexture(Wobject, Colormapable):
    """ BaseTexture(parent, data)
    
    Base texture class for visvis 2D and 3D textures.
    
    """
    
    def __init__(self, parent, data):
        
        # Check data first
        if not isinstance(data, np.ndarray):
            raise ValueError('Textures can only be described using Numpy arrays.')
        
        # Instantiate as wobject (after making "sure" this texture can be ok)
        Wobject.__init__(self, parent)
        Colormapable.__init__(self)
        
        # create texture (remember, this is an abstract class)
        self._texture1 = None
        
        # create glsl program for this texture...
        self._shader = shaders.Shader()
        
        # scale and translation transforms
        self._trafo_scale = Transform_Scale()
        self._trafo_trans = Transform_Translate()
        self.transformations.append(self._trafo_trans)
        self.transformations.append(self._trafo_scale)
    
    
    @property
    def shader(self):
        """ Get the shader object for the texture. This can
        be used to add code of your own and customize the vertex and
        fragment part of the shader.
        """
        return self._shader
    
    
    @DrawAfter
    def SetData(self, data):
        """ SetData(data)
        
        (Re)Set the data to display. If the data has the same shape
        as the data currently displayed, it can be updated very
        efficiently.
        
        If the data is an anisotripic array (vv.Aarray)
        the sampling and origin are (re-)applied.
        
        """
        
        # set data to texture
        self._SetData(data)
        
        # For 3D array, make vertex positions be re-calculated
        if hasattr(self, '_quads'):
            self._quads = None
        
        # Set fragment color part
        isColor = len(data.shape) > self._ndim
        if 'color' in self.shader.fragment.partNames:
            part = [shaders.SH_COLOR_SCALAR, shaders.SH_COLOR_RGB][isColor]
            self.shader.fragment.ReplacePart(part)
        
        # if Aarray, edit scaling and transform
        if hasattr(data,'sampling'):
            if self._ndim == 3:
                # Three dimensional
                self._trafo_scale.sx = data.sampling[2]
                self._trafo_scale.sy = data.sampling[1]
                self._trafo_scale.sz = data.sampling[0]
            elif self._ndim == 2:
                # Two dimensional
                self._trafo_scale.sx = data.sampling[1]
                self._trafo_scale.sy = data.sampling[0]
        if hasattr(data,'origin'):
            if self._ndim == 3:
                # Three dimensional
                self._trafo_trans.dx = data.origin[2]
                self._trafo_trans.dy = data.origin[1]
                self._trafo_trans.dz = data.origin[0]
            elif self._ndim == 2:
                # Two dimensional
                self._trafo_trans.dx = data.origin[1]
                self._trafo_trans.dy = data.origin[0]
    
    
    def _SetData(self, data):
        """ _SetData(data)
        
        Give reference to the raw data. For internal use. Inheriting
        classes can override this to store data in their own way and
        update the OpenGL textures accordingly.
        
        """
        self._texture1.SetData(data)
    
    
    def _GetData(self):
        """ _GetData()
        
        Get a reference to the raw data. For internal use. Can return None.
        
        """
        return self._texture1._dataRef
    
    
    def Refresh(self):
        """ Refresh()
        
        Refresh the data. If the numpy array was changed, calling this
        function will re-upload the data to OpenGl, making the change
        visible. This can be done efficiently.
        
        """
        data = self._GetData()
        if data is not None:
            self.SetData(data)
   
    
    def OnDestroyGl(self):
        # Clean up OpenGl resources.
        
        # remove texture from opengl memory
        if self._texture1 is not None:
            self._texture1.DestroyGl()
        
        # clear shader
        self._shader.DestroyGl()
        
        # remove colormap's texture from memory
        if hasattr(self, '_colormap'):
            self._colormap.DestroyGl()
    
    
    def OnDestroy(self):
        # Clean up any resources.
        if self._texture1 is not None:
            self._texture1.Destroy()
        self._shader.Destroy()
        if hasattr(self, '_colormap'):
            self._colormap.Destroy()
    
    
    def OnDrawFast(self):
        self.OnDraw(True)
    
    
    @PropWithDraw
    def interpolate():
        """ Get/Set whether to interpolate the image when zooming in
        (using linear interpolation).
        """
        def fget(self):
            return self._texture1._interpolate
        def fset(self, value):
            self._texture1._interpolate = bool(value)
            # Signal update
            self._texture1._uploadFlag = abs(self._texture1._uploadFlag)
        return locals()
    
    # Overload clim private methods so that the clim property
    # uses the clim of texture1
    def _GetClim(self):
        return self._texture1._clim
    def _SetClim(self, value):
        self._texture1._clim = value
    
    
    @DrawAfter
    def SetClim(self, *mima):
        """ SetClim(min, max)
        
        Set the contrast limits. Different than the property clim, this
        re-uploads the texture using different transfer functions. You should
        use this if your data has a higher contrast resolution than 8 bits.
        Takes a bit more time than clim though (which basically takes no
        time at all).
        
        """
        if len(mima)==0:
            # set default values
            data = self._GetData()
            if data is None:
                return
            mima = minmax(data)
        
        elif len(mima)==1:
            # a range was given
            mima = mima[0]
            
        # Set climref and clim
        self._texture1._climRef.Set(mima[0], mima[1])
        self._texture1._clim.Set(mima[0], mima[1])
        
        # Signal update, on next draw, it is uploaded again, using
        # the newly set climref.
        self._texture1._uploadFlag = abs(self._texture1._uploadFlag)




class Texture2D(BaseTexture):
    """ Texture2D(parent, data)
    
    A data type that represents structured data in
    two dimensions (an image). Supports grayscale, RGB,
    and RGBA images.
    
    Texture2D objects can be created with the function vv.imshow().
    
    """
    
    def __init__(self, parent, data):
        BaseTexture.__init__(self, parent, data)
        self._ndim = 2
        
        # create texture and set data
        self._texture1 = TextureObjectToVisualize(2, data)
        
        # init shader
        self._InitShader()
        
        # set data
        self.SetData(data)
        
        # init antialiasing
        self.aa = 2
    
    
    def _InitShader(self):
        
        # Add components of shaders
        self.shader.vertex.Clear()
        self.shader.fragment.Clear()
        self.shader.fragment.AddPart(shaders.SH_2F_BASE)
        self.shader.fragment.AddPart(shaders.SH_2F_AASTEPS_2)
        self.shader.fragment.AddPart(shaders.SH_COLOR_SCALAR)
        
        def uniform_shape():
            shape = self._texture1._shape[:2] # as in opengl
            return [float(s) for s in reversed(list(shape))]
        def uniform_extent():
            data = self._texture1._dataRef # as original array
            shape = reversed(data.shape[:2])
            if hasattr(data, 'sampling'):
                sampling = reversed(data.sampling[:2])
            else:
                sampling = [1.0 for s in range(2)]
            
            del data
            return [s1*s2 for s1, s2 in zip(shape, sampling)]
        
        # Set some uniforms
        self.shader.SetStaticUniform('colormap', self._colormap)
        self.shader.SetStaticUniform('shape', uniform_shape)
        self.shader.SetStaticUniform('scaleBias', self._texture1._ScaleBias_get)
        self.shader.SetStaticUniform('extent', uniform_extent)
        self.shader.SetStaticUniform('aakernel', self._CreateAaKernel)
    
    
    def _CreateAaKernel(self):
        """ Create kernel values to use in the aa program.
        Returns 4 element list which should be applied using the
        following indices: 3 2 1 0 1 2 3
        
        We use a Lanczos kernel: a windowed sinc function.
        (In previous versions we used a Gaussian, but lanczos filtering
        has better frequency repsonse; images stay more crysp.)
        
        Thanks to Nicolas Rougier for pointing out that the Gaussian filter
        is not the best for aa.
        
        """
        
        figure = self.GetFigure()
        axes = self.GetAxes()
        if not figure or not axes:
            return 1,0,0,0
        
        # determine relative kernel size
        w,h = figure.position.size
        cam = axes.camera
        sx = (1.0 / abs(axes.daspectNormalized[0]*cam._zoom) ) / w
        sy = (1.0 / abs(axes.daspectNormalized[1]*cam._zoom) ) / h
        
        # For cutoff frequency we take average of both dimensons.
        B = 2.0/(sx+sy)
        
        # Define sinc function
        def sinc(x):
            if x==0.0:
                return 1.0
            else:
                return float( np.sin(x) / x )
        
        # Calculate kernel values
        a = 3.0 # Number of side lobes of sync to take into account.
        k = [0,0,0,0]
        for t in range(4):
            k[t] = 2*B * sinc(2*B*t)*sinc(2*B*t/a)
        
        # Normalize (take kenel size into account)
        if self.aa == 1:
            l = k[0] + 2*k[1]
        elif self.aa == 2:
            l = k[0] + 2*k[1] + 2*k[2]
        elif self.aa == 3:
            l = k[0] + 2*k[1] + 2*k[2] + 2*k[3]
        else:
            l = k[0]
        k = [float(e)/l for e in k]
        
        # Done
        return k
    
    
    def OnDrawShape(self, clr):
        # Implementation of the OnDrawShape method.
        gl.glColor(clr[0], clr[1], clr[2], 1.0)
        self._DrawQuads()
    

    def OnDraw(self, fast=False):
        # Draw the texture.
        
        # set color to white, otherwise with no shading, there is odd scaling
        gl.glColor3f(1.0,1.0,1.0)
        
        # draw texture also from beneeth
        #gl.glCullFace(gl.GL_FRONT_AND_BACK)
        
        # Enable texture, so that it has a corresponding OpenGl texture.
        # Binding is done by the shader
        self._texture1.Enable(-1)
        self.shader.SetUniform('texture', self._texture1)
        
        # _texture._shape is a good indicator of a valid texture
        if not self._texture1._shape:
            return
        
        if self.shader.isUsable and self.shader.hasCode:
            # fragment shader on -> anti-aliasing
            self.shader.Enable()
        else:
            # Fixed funcrion pipeline
            self.shader.EnableTextureOnly('texture')
        
        # do the drawing!
        self._DrawQuads()
        gl.glFlush()
        
        # clean up
        self.shader.Disable()
    
    
    def _DrawQuads(self):
        """ Draw the quads of the texture.
        This is done in a seperate method to reuse code in
        OnDraw() and OnDrawShape().
        """
        if not self._texture1._shape:
            return
        
        # The -0.5 offset is to center pixels/voxels. This works correctly
        # for anisotropic data.
        #x1, x2 = -0.5, self._texture1._shape[1]-0.5
        #y2, y1 = -0.5, self._texture1._shape[0]-0.5
        x1, x2 = -0.5, self._texture1._dataRef.shape[1]-0.5
        y2, y1 = -0.5, self._texture1._dataRef.shape[0]-0.5
        
        # draw
        gl.glBegin(gl.GL_QUADS)
        gl.glTexCoord2f(0,0); gl.glVertex3d(x1, y2, 0.0)
        gl.glTexCoord2f(1,0); gl.glVertex3d(x2, y2, 0.0)
        gl.glTexCoord2f(1,1); gl.glVertex3d(x2, y1, 0.0)
        gl.glTexCoord2f(0,1); gl.glVertex3d(x1, y1, 0.0)
        gl.glEnd()
    
    
    def _GetLimits(self):
        """ Get the limits in world coordinates between which the object exists.
        """
        
        # Obtain untransformed coords
        shape = self._texture1._dataRef.shape
        #shape = self._texture1._shape
        x1, x2 = -0.5, shape[1]-0.5
        y1, y2 = -0.5, shape[0]-0.5
        z1, z2 = 0, 0
        
        # There we are
        return Wobject._GetLimits(self, x1, x2, y1, y2, z1, z2)
    
    
    @PropWithDraw
    def aa():
        """ Get/Set anti aliasing quality.
          * 0 or False for no anti aliasing
          * 1 for anti aliasing using 3-element kernel.
          * 2 for anti aliasing using 5-element kernel.
          * 3 for anti aliasing using 7-element kernel.
        
        Higher numbers result in better quality anti-aliasing,
        but may be slower on older hardware.
        
        Note that in previous versions of visvis, this property influenced
        the *amount* of aliasing. We now use a better kernel (Lanczos instead
        of Gaussian), such that the amount can be fixed without negatively
        affecting the visualization.
        """
        def fget(self):
            return self._aa
        def fset(self, value):
            if not value:
                value = 0
            if not isinstance(value, (int,float)):
                raise ValueError("Texture2D.aa: value should be 0,1,2,3.")
            elif value < 0 or value > 3:
                raise ValueError("Texture2D.aa: value should be 0,1,2,3.")
            else:
                # Store
                self._aa = int(value)
                # Get aa program part
                M = [   shaders.SH_2F_AASTEPS_0, shaders.SH_2F_AASTEPS_1,
                        shaders.SH_2F_AASTEPS_2, shaders.SH_2F_AASTEPS_3]
                aa_steps = M[self._aa]
                # Apply
                self.shader.fragment.AddOrReplace(aa_steps)
        return locals()


class Texture3D(BaseTexture):
    """ Texture3D(parent, data, renderStyle='mip')
    
    A data type that represents structured data in three dimensions (a volume).
    
    If the drawing hangs, your video drived decided to render in
    software mode. This is unfortunately (as far as I know) not possible
    to detect programatically. It might help if your data is shaped a
    power of 2. The mip renderer is the 'easiest' for most systems to render.
    
    Texture3D objects can be created with the function vv.volshow().
    
    """
    
    def __init__(self, parent, data, renderStyle='mip'):
        BaseTexture.__init__(self, parent, data)
        self._ndim = 3
        
        # create texture
        self._texture1 = TextureObjectToVisualize(3, data)
        
        # Init vertex and fragment shader
        self._InitShader()
        
        # set data
        self.SetData(data)
        
        # Init deform
        self._deformation = None
        
        # init interpolation
        self._texture1._interpolate = True # looks so much better
        
        # init iso shader param
        self._isoThreshold = 0.0
        self._stepRatio = 1.0
        
        # Attribute to store array of quads (vertices and texture coords)
        self._quads = None
        # Also store daspect, if this changes quads should be recalculated
        self._daspectStored = (1,1,1)
        self._qcountStored = -1
        
        # Set renderstyle
        self.renderStyle = renderStyle
    
    
    def _InitShader(self):
        
        # Add components of shaders
        self.shader.vertex.Clear()
        self.shader.vertex.AddPart(shaders.SH_3V_BASE)
        #
        self.shader.fragment.Clear()
        self.shader.fragment.AddPart(shaders.SH_3F_BASE)
        self.shader.fragment.AddPart(shaders.SH_3F_CALCSTEPS)
        self.shader.fragment.AddPart(shaders.SH_3F_STYLE_MIP)
        self.shader.fragment.AddPart(shaders.SH_COLOR_SCALAR)
        
        
        def uniform_shape():
            shape = self._texture1._shape[:3] # as in opengl
            return [float(s) for s in reversed(list(shape))]
        def uniform_th():
            ran = self._texture1._climRef.range
            if ran==0:
                ran = 1.0
            th = (self._isoThreshold - self._texture1._climRef.min ) / ran
            return th
        def uniform_extent():
            data = self._texture1._dataRef
            shape = reversed(data.shape[:3])
            if hasattr(data, 'sampling'):
                sampling = reversed(data.sampling[:3])
            else:
                sampling = [1.0 for s in range(3)]
            del data
            return [s1*s2 for s1, s2 in zip(shape, sampling)]
        
        # Set some uniforms
        self.shader.SetStaticUniform('colormap', self._colormap)
        self.shader.SetStaticUniform('shape', uniform_shape)
        self.shader.SetStaticUniform('th', uniform_th)
        self.shader.SetStaticUniform('scaleBias', self._texture1._ScaleBias_get)
        self.shader.SetStaticUniform('extent', uniform_extent)
        
        # Set lighting for iso renderer
        self.shader.SetStaticUniform('ambient', [0.7,0.7,0.7,1.0])
        self.shader.SetStaticUniform('diffuse', [0.7,0.7,0.7,1.0])
        self.shader.SetStaticUniform('specular', [0.3,0.3,0.3,1.0])
        self.shader.SetStaticUniform('shininess', 50.0)
        # And number of surface samples
        self.shader.SetStaticUniform('maxIsoSamples', 3)
    
    
    def OnDrawShape(self, clr):
        # Implementation of the OnDrawShape method.
        gl.glColor(clr[0], clr[1], clr[2], 1.0)
        self._DrawQuads()
    
    
    def OnDraw(self, fast=False):
        # Draw the texture.
        
        # Enable texture, so that it has a corresponding OpenGl texture.
        # Binding is done by the shader
        self._texture1.Enable(-1) # -1 means do not bind right now
        self.shader.SetUniform('texture', self._texture1)
        
        # _texture._shape is a good indicator of a valid texture
        if not self._texture1._shape:
            return
        
        
        # Prepare by setting things to their defaults. This might release some
        # memory so result in a bigger chance that the shader is run in
        # hardware mode. On ATI, the line and point smoothing should be off
        # if you want to use gl_FragCoord. (Yeah, I do not see the connection
        # either...)
        gl.glPointSize(1)
        gl.glLineWidth(1)
        gl.glDisable(gl.GL_LINE_STIPPLE)
        gl.glDisable(gl.GL_LINE_SMOOTH)
        gl.glDisable(gl.GL_POINT_SMOOTH)
        
        # only draw front-facing parts
        gl.glEnable(gl.GL_CULL_FACE)
        gl.glCullFace(gl.GL_BACK)
        
        # Set step ratio
        if fast:
            self.shader.SetUniform('stepRatio', self._stepRatio*0.5)
        else:
            self.shader.SetUniform('stepRatio', float(self._stepRatio))
        
        # Set right number of lights, or disabled light stuff
        if self.shader.fragment.HasPart('litvoxel'):
            self._EnsureRightNumberOfLights(self.GetAxes(), self.shader)
        else:
            self.shader.vertex.AddOrReplace(shaders.SH_NLIGHTS_0)
            self.shader.fragment.AddOrReplace(shaders.SH_NLIGHTS_0)
        
        
        if self.shader.isUsable and self.shader.hasCode:
            # turn glsl shader on
            ok = self.shader.Enable()
            if (not ok) and (self.renderStyle != 'mip'):
                print('Texture3D detected shader problem; ' +
                    'reverting render style from %s to MIP.' % self.renderStyle)
                self.renderStyle = 'mip'
                self.shader.Enable()
        else:
            # Fixed function pipeline, but does not make sense
            self.shader.EnableTextureOnly('texture')
        
        
        # do the actual drawing
        self._DrawQuads()
        
        # clean up
        gl.glFlush()
        self.shader.Disable()
        #
        gl.glDisable(gl.GL_CULL_FACE)
        gl.glEnable(gl.GL_LINE_SMOOTH)
        gl.glEnable(gl.GL_POINT_SMOOTH)
    
    
    def _EnsureRightNumberOfLights(self, axes, shader):
        
        # Check number of lights in axes
        nlights = 1
        for i in range(1, 7): # len(axes.lights)):
            if axes.lights[i].isOn:
                nlights = i+1
        
        # Define shader part to use
        M = [ shaders.SH_NLIGHTS_1, shaders.SH_NLIGHTS_2, shaders.SH_NLIGHTS_3,
              shaders.SH_NLIGHTS_4, shaders.SH_NLIGHTS_5, shaders.SH_NLIGHTS_6,
              shaders.SH_NLIGHTS_7, shaders.SH_NLIGHTS_8, ]
        SH_LIGHTS = M[nlights-1]
        
        # Ensure that the right light shaderpart is selected
        if not shader.vertex.HasPart(SH_LIGHTS):
            shader.vertex.AddOrReplace(SH_LIGHTS)
        if not shader.fragment.HasPart(SH_LIGHTS):
            shader.fragment.AddOrReplace(SH_LIGHTS)
    
    
    def _quadPartitionCount(self, camera):
        """ Select the density of the vertices for the cube to render.
        Higher field of view yields higher partition-count.
        """
        if hasattr(camera, 'fov'):
            fov = camera.fov
        else:
            return 0
        
        if fov <= 10:
            return 0  # 24 vertices
        elif fov <= 20:
            return 1  # 96 vertices
        elif fov <= 40:
            return 2  # 384 vertices
        elif fov <= 80:
            return 3  # 1536 vertices
        else:
            return 4  # 6144 vertices
    
    
    def _CreateQuads(self):
        
        axes = self.GetAxes()
        if not axes:
            return
        
        # Store daspect so we can detect it changing
        self._daspectStored = axes.daspect
        self._qcountStored = self._quadPartitionCount(axes.camera)
        
        # Note that we could determine the world coordinates and use
        # them directly here. However, the way that we do it now (using
        # the transformations) is to be preferred, because that way the
        # transformations are applied via the ModelView matrix stack,
        # and can easily be made undone in the raycaster.
        # The -0.5 offset is to center pixels/voxels. This works correctly
        # for anisotropic data.
        #shape = self._texture1._shape
        shape = self._texture1._dataRef.shape
        x0, x1 = -0.5, shape[2]-0.5
        y0, y1 = -0.5, shape[1]-0.5
        z0, z1 = -0.5, shape[0]-0.5
        
        # prepare texture coordinates
        t0, t1 = 0, 1
        
        # I previously swapped coordinates to make sure the right faces
        # were frontfacing. Now I apply culling to achieve the same
        # result in a better way.
        
        # using glTexCoord* is the same as glMultiTexCoord*(GL_TEXTURE0)
        # Therefore we need to bind the base texture to 0.
        
        # So we draw the six planes of the cube (well not a cube,
        # a 3d rectangle thingy). The inside is only rendered if the
        # vertex is facing front, so only 3 planes are rendered at a
        # time...
        
        
        # Define the 8 corners of the cube.
        tex_coord0, ver_coord0 = Pointset(3), Pointset(3)
        # bottom
        tex_coord0.append((t0,t0,t0)); ver_coord0.append((x0, y0, z0)) # 0
        tex_coord0.append((t1,t0,t0)); ver_coord0.append((x1, y0, z0)) # 1
        tex_coord0.append((t1,t1,t0)); ver_coord0.append((x1, y1, z0)) # 2
        tex_coord0.append((t0,t1,t0)); ver_coord0.append((x0, y1, z0)) # 3
        # top
        tex_coord0.append((t0,t0,t1)); ver_coord0.append((x0, y0, z1)) # 4
        tex_coord0.append((t0,t1,t1)); ver_coord0.append((x0, y1, z1)) # 5
        tex_coord0.append((t1,t1,t1)); ver_coord0.append((x1, y1, z1)) # 6
        tex_coord0.append((t1,t0,t1)); ver_coord0.append((x1, y0, z1)) # 7
        
        # Unwrap the vertices. 4 vertices per side = 24 vertices
        # Warning: dont mess up the list with indices; theyre carefully
        # chosen to be front facing.
        tex_coord, ver_coord = Pointset(3), Pointset(3)
        for i in [0,1,2,3, 4,5,6,7, 3,2,6,5, 0,4,7,1, 0,3,5,4, 1,7,6,2]:
            tex_coord.append(tex_coord0[i])
            ver_coord.append(ver_coord0[i])
        
        # Function to partition each quad in four smaller quads
        def partition(tex_coord1, ver_coord1):
            tex_coord2, ver_coord2 = Pointset(3), Pointset(3)
            for iQuad in range(int(len(tex_coord1)/4)):
                io = iQuad * 4
                for i1 in range(4):
                    for i2 in range(4):
                        i3 = (i1 + i2)%4
                        tex_coord2.append( 0.5*(tex_coord1[io+i1] + tex_coord1[io+i3]) )
                        ver_coord2.append( 0.5*(ver_coord1[io+i1] + ver_coord1[io+i3]) )
            #print('partition from %i to %i vertices' % (len(tex_coord1), len(tex_coord2)))
            return tex_coord2, ver_coord2
        
        # Partition quads in smaller quads?
        for iter in range(self._qcountStored):
            tex_coord, ver_coord = partition(tex_coord, ver_coord)
        
        # Store quads data
        self._quads = tex_coord, ver_coord
    
    
    def _DrawQuads(self):
        """ Draw the quads of the texture.
        This is done in a seperate method to reuse code in
        OnDraw() and OnDrawShape().
        """
        
        # Get axes
        axes = self.GetAxes()
        if not axes:
            return
        
        # should we draw?
        if not self._texture1._shape:
            return
        
        # should we create quads?
        if (    (not self._quads) or
                (self._daspectStored != axes.daspect) or
                (self._qcountStored != self._quadPartitionCount(axes.camera))
            ):
            self._CreateQuads()
        
        # get data
        tex_coord, ver_coord = self._quads
        
        # Set culling (take data aspect into account!)
        tmp = 1
        for i in axes.daspect:
            if i<0:
                tmp *= -1
        gl.glFrontFace({1:gl.GL_CW, -1:gl.GL_CCW}[tmp])
        gl.glEnable(gl.GL_CULL_FACE)
        gl.glCullFace(gl.GL_BACK)
        
        # init vertex and texture array
        gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
        gl.glEnableClientState(gl.GL_TEXTURE_COORD_ARRAY)
        gl.glVertexPointerf(ver_coord.data)
        gl.glTexCoordPointerf(tex_coord.data)
        
        # draw
        gl.glDrawArrays(gl.GL_QUADS, 0, len(tex_coord))
        
        # disable vertex array
        gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
        gl.glDisableClientState(gl.GL_TEXTURE_COORD_ARRAY)
        #
        gl.glDisable(gl.GL_CULL_FACE)
    
    
    def _GetLimits(self):
        """ Get the limits in world coordinates between which the object exists.
        """
        
        # Obtain untransformed coords
        shape = self._texture1._dataRef.shape
        x1, x2 = -0.5, shape[2]-0.5
        y1, y2 = -0.5, shape[1]-0.5
        z1, z2 = -0.5, shape[0]-0.5
        
        # There we are
        return Wobject._GetLimits(self, x1, x2, y1, y2, z1, z2)
    
    
    @PropWithDraw
    def renderStyle():
        """ Get/Set the render style to render the volumetric data:
          * MIP: maximum intensity projection. Shows the voxel with the
            maxiumum value.
          * ISO: isosurface rendering. Casts ray until value is above
            threshold. Ligthing is calculated at that voxel. Use together
            with isoThreshold property.
          * RAY: ray casting. All voxels along the ray contribute to final
            color, weighted by the alpha value.
          * EDGERAY: ray casting in which alpha is scaled with the gradient
            magnitude.
          * LITRAY: ray casting in which all voxels are lit. Most pretty and
            most demanding for GPU.
        
        Notes
        =====
        MIP and EDGERAY usually work out of the box. ISO requires playing
        with the isoThreshold property. RAY and LITRAY require playing with
        the alpha channel using the ColormapEditor wibject.
        
        If drawing takes really long, your system renders in software
        mode. Try rendering data that is shaped with a power of two. This
        helps on some cards.
        
        You can also create your own render style by modyfying the GLSL code.
        See core/shaders_src.py and the ShaderCode class for more info.
        """
        def fget(self):
            return self._renderStyle
        def fset(self, style):
            style = style.lower()
            shader = self.shader.fragment
            # Set color (done automatically!)
            # sh_color = shaders.SH_COLOR_SCALAR
            # if 'color' in style or 'rgb' in style:
            #     sh_color = shaders.SH_COLOR_RGB
            # elif 'nocmap' in style:
            #     sh_color = shaders.SH_COLOR_SCALAR_NOCMAP
            styleStrict = style.replace('rgb','').replace('color','')
            # Set render style
            if 'mip' == styleStrict:
                shader.ReplacePart(shaders.SH_3F_STYLE_MIP)
                shader.RemovePart('litvoxel')
            elif 'iso' == styleStrict:
                lf = shaders.SH_3F_LITVOXEL
                shader.ReplacePart(shaders.SH_3F_STYLE_ISO)
                shader.AddOrReplace(lf, after='renderstyle')
            elif 'ray' == styleStrict:
                shader.ReplacePart(shaders.SH_3F_STYLE_RAY)
                shader.RemovePart('litvoxel')
            elif 'edgeray' == styleStrict:
                shader.ReplacePart(shaders.SH_3F_STYLE_EDGERAY)
                shader.RemovePart('litvoxel')
            elif 'litray' == styleStrict:
                lf = shaders.SH_3F_LITVOXEL
                shader.ReplacePart(shaders.SH_3F_STYLE_LITRAY)
                shader.AddOrReplace(lf, after='renderstyle')
            else:
                raise ValueError("Unknown render style in Texture3d.renderstyle.")
            # Store style and set color
            self._renderStyle = style
            # self._fragmentShader.ReplacePart('color', sh_color)
        return locals()
    
    
    @PropWithDraw
    def isoThreshold():
        """ Get/Set the isothreshold value used in the iso renderer.
        """
        def fget(self):
            return self._isoThreshold
        def fset(self, value):
            # make float
            value = float(value)
            # store
            self._isoThreshold = value
        return locals()


def _interpolateMotionData(self, ii, ww):
    # Get data via interpolation
    data = None
    for i,w in zip(ii, ww):
        if w != 0:
            if data is None:
                data = self._motionData[i] * w
            else:
                data += self._motionData[i] * w
    return data


class MotionTexture2D(Texture2D, MotionMixin):
    """ MotionTexture2D(parent, data)
    
    A data type that represents a 2D texture in motion.
    The given data must be a list of the images that should be shown.
    
    The motionIndex (i.e. time) can also be in between two
    images, in which case interpolation is applied (default linear).
    
    """
    
    def __init__(self, parent, data, *args, **kwargs):
        Texture2D.__init__(self, parent, data[0], *args, **kwargs)
        MotionMixin.__init__(self)
        
        # Store motion data
        self._motionData = [d for d in data]
    
    def _GetMotionCount(self):
        """ _getMotionCount()
        
        Get the number of textures that make up the motion texture.
        
        """
        return len(self._motionData)
  
    def _SetMotionIndex(self, index, ii, ww):
        """ _setMotionIndex(index, ii, ww)
        
        Make the right child visible.
        
        """
        data = _interpolateMotionData(self, ii, ww)
        self.SetData(data)


class MotionTexture3D(Texture3D, MotionMixin):
    """ MotionTexture2D(parent, data)
    
    A data type that represents a 3D texture in motion.
    The given data must be a list of the volumes that should be shown.
    
    The motionIndex (i.e. time) can also be in between two
    images, in which case interpolation is applied (default linear).
    
    Note that this can be rather slow. For faster display of
    multiple images, one can also use the MotionDataContainer.
    
    """
    
    def __init__(self, parent, data, *args, **kwargs):
        Texture3D.__init__(self, parent, data[0], *args, **kwargs)
        MotionMixin.__init__(self)
        
        # Store motion data
        self._motionData = [d for d in data]
    
    def _GetMotionCount(self):
        """ _getMotionCount()
        
        Get the number of textures that make up the motion texture.
        
        """
        return len(self._motionData)
  
    def _SetMotionIndex(self, index, ii, ww):
        """ _setMotionIndex(index, ii, ww)
        
        Make the right child visible.
        
        """
        data = _interpolateMotionData(self, ii, ww)
        self.SetData(data)


class MultiTexture3D(Texture3D):
    """ MultiTexture3D(parent, data1, data2)
    
    This is an example of what multi-texturing would look like
    in Visvis. Not tested.
    
    """
    
    def __init__(self, parent, data1, data2):
        Texture3D.__init__(self, parent, data1)
        
        # create second texture and set data
        self._texture2 = TextureObject(gl.GL_TEXTURE_3D)
        self.SetData(data2)
    
    
    def OnDraw(self, fast=False):
        # Draw the texture.
        
        # Enable texture, so that it has a corresponding OpenGl texture.
        # Binding is done by the shader
        self._texture1.Enable(-1)
        self.shader.SetUniform('texture', self._texture1)
        self.shader.SetUniform('texture2', self._texture2)
        
        # _texture._shape is a good indicator of a valid texture
        if not self._texture1._shape:
            return
        
        # Prepare by setting things to their defaults. This might release some
        # memory so result in a bigger chance that the shader is run in
        # hardware mode. On ATI, the line and point smoothing should be off
        # if you want to use gl_FragCoord. (Yeah, I do not see the connection
        # either...)
        gl.glPointSize(1)
        gl.glLineWidth(1)
        gl.glDisable(gl.GL_LINE_STIPPLE)
        gl.glDisable(gl.GL_LINE_SMOOTH)
        gl.glDisable(gl.GL_POINT_SMOOTH)
        
        # only draw front-facing parts
        gl.glEnable(gl.GL_CULL_FACE)
        gl.glCullFace(gl.GL_BACK)
        
        # Set step ratio
        if fast:
            self.shader.SetUniform('stepRatio', self._stepRatio*0.5)
        else:
            self.shader.SetUniform('stepRatio', float(self._stepRatio))
        
        
        # fragment shader on
        if self.shader.isUsable and self.shader.hasCode:
            # turn glsl shader on
            self.shader.Enable()
        else:
            pass # glsl not available, or someone cleared the code
            
        # do the actual drawing
        self._DrawQuads()
        
        # clean up
        gl.glFlush()
        self.shader.Disable()
        #
        gl.glDisable(gl.GL_CULL_FACE)
        gl.glEnable(gl.GL_LINE_SMOOTH)
        gl.glEnable(gl.GL_POINT_SMOOTH)


    def OnDestroyGl(self):
        # Clean up OpenGl resources.
        
        # remove texture from opengl memory
        self._texture1.DestroyGl()
        self._texture2.DestroyGl()
        
        # clear shaders
        self._shader.DestroyGl()
        
        # remove colormap's texture from memory
        if hasattr(self, '_colormap'):
            self._colormap.DestroyGl()
    
    
    def OnDestroy(self):
        # Clean up any resources.
        self._texture1.Destroy()
        self._texture2.Destroy()
        if hasattr(self, '_colormap'):
            self._colormap.Destroy()



#!/usr/bin/env python

import visvis as vv
app = vv.use()

# Get green channel of lena image
im = vv.imread('astronaut.png')[:,:,1]

# Make 4 subplots with different colormaps
cmaps = [vv.CM_GRAY, vv.CM_JET, vv.CM_SUMMER, vv.CM_HOT]
for i in range(4):
    a = vv.subplot(2,2,i+1)
    t = vv.imshow(im, clim=(0,255))
    a.axis.visible = 0
    t.colormap = cmaps[i]
    vv.colorbar()

app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def cla():
    """ cla()
    
    Clear the current axes.
    
    """
    a = vv.gca()
    a.Clear()
    return a


if __name__ == '__main__':
    # Show plot
    vv.plot([1,2,3,1,4])
    # Clear the plot
    vv.cla()



# -*- coding: utf-8 -*-
#   Copyright (C) 2012, Almar Klein
#   This code is subject to the (new) BSD license:
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the <organization> nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

""" Package vvmovie (visvis-movie)

All submodules have been designed to be work independent of each-other
(except the image2avi module, which requires the images2ims module).

Provides the following functions:
  * readGif & writeGif  -> a movie stored as animated GIF
  * readSwf & writeSwf  -> a movie stored as shockwave flash
  * readAvi & writeAvi  -> a movie stored as compressed video
  * readIms & writeIms  -> a movie stored as a series of images

Two additional functions are provided for ease of use, that call
the right function depending on the used file extension:
  * movieRead
  * movieWrite

More information about compression and limitations:
  * GIF. Requires PIL. Animated GIF applies a color-table of maximal
    256 colors and applies poor compression. It's widely applicable though.
  * SWF. Provides lossless storage of movie frames with good (ZLIB)
    compression. Reading of SWF files is limited to images stored using ZLIB
    compression (no JPEG files). Requires no external libraries.
  * AVI. Requires ffmpeg. Most Linux can obtain it using their package
    manager. Windows users can use the installer at the visvis website.
    Provides excelent mpeg4 (or any other supported by ffmpeg) compression.
    Not intended for reading very large movies.
  * IMS. Requires PIL. Quality depends on the used image type. Use png for
    lossless compression and jpg otherwise.

"""

import os, time, warnings

# Python 3 needs absolute import, which makes that this package
# cannot be a subpackage anymore. We cannot use the dot-notation,
# because that doesnt work on Python 2.
from visvis.vvmovie.images2gif import readGif, writeGif
from visvis.vvmovie.images2swf import readSwf, writeSwf
from visvis.vvmovie.images2avi import readAvi, writeAvi
from visvis.vvmovie.images2ims import readIms, writeIms

videoTypes = ['AVI', 'MPG', 'MPEG', 'MOV', 'FLV']
imageTypes = ['JPG', 'JPEG', 'PNG', 'TIF', 'TIFF', 'BMP']


def movieWrite(filename, images, duration=0.1, repeat=True, **kwargs):
    """ movieWrite(fname, images, duration=0.1, repeat=True, **kwargs)
    
    Write the movie specified in images to GIF, SWF, AVI/MPEG, or a series
    of images (PNG,JPG,TIF,BMP).
    
    General parameters
    ------------------
    filename : string
       The name of the file to write the image to. For a series of images,
        the `*` wildcard can be used.
    images : list
        Should be a list consisting of PIL images or numpy arrays.
        The latter should be between 0 and 255 for integer types,
        and between 0 and 1 for float types.
    duration : scalar
        The duration for all frames. For GIF and SWF this can also be a list
        that specifies the duration for each frame. (For swf the durations
        are rounded to integer amounts of the smallest duration.)
    repeat : bool or integer
        Can be used in GIF and SWF to indicate that the movie should
        loop. For GIF, an integer can be given to specify the number of loops.
    
    Special GIF parameters
    ----------------------
    dither : bool
        Whether to apply dithering
    nq : integer
        If nonzero, applies the NeuQuant quantization algorithm to create
        the color palette. This algorithm is superior, but slower than
        the standard PIL algorithm. The value of nq is the quality
        parameter. 1 represents the best quality. 10 is in general a
        good tradeoff between quality and speed. When using this option,
        better results are usually obtained when subRectangles is False.
    subRectangles : False, True, or a list of 2-element tuples
        Whether to use sub-rectangles. If True, the minimal rectangle that
        is required to update each frame is automatically detected. This
        can give significant reductions in file size, particularly if only
        a part of the image changes. One can also give a list of x-y
        coordinates if you want to do the cropping yourself. The default
        is True.
    dispose : int
        How to dispose each frame. 1 means that each frame is to be left
        in place. 2 means the background color should be restored after
        each frame. 3 means the decoder should restore the previous frame.
        If subRectangles==False, the default is 2, otherwise it is 1.
    
    Special AVI/MPEG parameters
    ---------------------------
    encoding : {'mpeg4', 'msmpeg4v2', ...}
        The encoding to use. Hint for Windows users: the 'msmpeg4v2' codec
        is natively supported on Windows.
    inputOptions : string
        See the documentation of ffmpeg
    outputOptions : string
        See the documentation of ffmpeg
    
    Notes for writing a series of images
    ------------------------------------
    If the filenenumber contains an asterix, a sequence number is introduced
    at its location. Otherwise the sequence number is introduced right before
    the final dot. To enable easy creation of a new directory with image
    files, it is made sure that the full path exists.
    
    Notes for writing AVI/MPEG
    --------------------------
    Writing AVI requires the "ffmpeg" application:
      * Most linux users can install it using their package manager.
      * There is a windows installer on the visvis website.
    
    Notes on compression and limitations
    ------------------------------------
      * GIF: Requires PIL. Animated GIF applies a color-table of maximal
        256 colors. It's widely applicable though. Reading back GIF images
        can be problematic due to the applied color reductions and because
        of problems with PIL.
      * SWF: Provides lossless storage of movie frames with good (ZLIB)
        compression. Reading of SWF files is limited to images stored using
        ZLIB compression. Requires no external libraries.
      * AVI: Requires ffmpeg. Provides excelent mpeg4 (or any other supported
        by ffmpeg) compression. Not intended for reading very large movies.
      * IMS: Requires PIL. Quality depends on the used image type. Use png for
        lossless compression and jpg otherwise.
    
    """
    
    warnings.warn('Visvis movieWrite() function and vvmovie module are supersceded by the imageio library.')
    
    # Test images
    if not isinstance(images, (tuple, list)):
        raise ValueError("Images should be a tuple or list.")
    if not images:
        raise ValueError("List of images is empty.")
    
    # Get extension
    EXT = os.path.splitext(filename)[1]
    EXT = EXT[1:].upper()
    
    # Start timer
    t0 = time.time()
    
    # Write
    if EXT == 'GIF':
        writeGif(filename, images, duration, repeat, **kwargs)
    elif EXT == 'SWF':
        writeSwf(filename, images, duration, repeat, **kwargs)
    elif EXT in videoTypes:
        writeAvi(filename, images, duration, **kwargs)
    elif EXT in imageTypes:
        writeIms(filename, images, **kwargs)
    else:
        raise ValueError('Given file extension not valid: '+EXT)
    
    # Stop timer
    t1 = time.time()
    dt = t1-t0
    
    # Notify
    print("Wrote %i frames to %s in %1.2f seconds (%1.0f ms/frame)" %
                        (len(images), EXT, dt, 1000*dt/len(images)) )


def movieRead(filename, asNumpy=True, **kwargs):
    """ movieRead(filename, asNumpy=True)
    
    Read the movie from GIF, SWF, AVI (or MPG), or a series of images (PNG,
    JPG,TIF,BMP).
    
    Parameters
    ----------
    filename : string
        The name of the file that contains the movie. For a series of images,
        the `*` wildcard can be used.
    asNumpy : bool
        If True, returns a list of numpy arrays. Otherwise return
        a list if PIL images.
    
    Notes
    ------
    Reading AVI requires the "ffmpeg" application:
      * Most linux users can install it using their package manager
      * There is a windows installer on the visvis website
    
    """
    
    warnings.warn('Visvis movieRead() function and vvmovie module are supersceded by the imageio library.')
    
    # Get extension
    EXT = os.path.splitext(filename)[1]
    EXT = EXT[1:].upper()
    
    # Start timer
    t0 = time.time()
    
    # Write
    if EXT == 'GIF':
        images = readGif(filename, asNumpy, **kwargs)
    elif EXT == 'SWF':
        images = readSwf(filename,  asNumpy, **kwargs)
    elif EXT in videoTypes:
        images = readAvi(filename,  asNumpy, **kwargs)
    elif EXT in imageTypes:
        images = readIms(filename,  asNumpy, **kwargs)
    else:
        raise ValueError('Given file extension not valid: '+EXT)
    
    # Stop timer
    t1 = time.time()
    dt = t1-t0
    
    # Notify
    if images:
        print("Read %i frames from %s in %1.2f seconds (%1.0f ms/frame)" %
                    (len(images), EXT, dt, 1000*dt/len(images)) )
    else:
        print("Could not read any images.")
    
    # Done
    return images




#!/usr/bin/env python

import visvis as vv
app = vv.use()

im = vv.imread('astronaut.png')
im = im[:-1,:-1] # make not-power-of-two (to test if video driver is capable)
print(im.shape)

t = vv.imshow(im)
t.aa = 2 # more anti-aliasing (default=1)
t.interpolate = True # interpolate pixels

app.Run()



#!/usr/bin/env python
""" This example shows hoe the GLSL program for a 2D texture
can be modified to achieve all kinds of graphic effects.
In this example we will demonstrate showing the edges of an image.
Also see the unsharpMasking example.
"""

import visvis as vv

# First define our part of the shading code
# The '>>' Denote what piece of code we want to replace. We replace the
# standard texture lookup for one that samples the derivatives, and use them
# to calculate the gradient magnitude.
#
# Note that in contrast to the unsharp masking example, this example leaves
# the anti-aliasing in place, so we can zoom out without the edges flickering.
SH_2F_EDGE = vv.shaders.ShaderCodePart('edge', 'gradient magnitude',
"""
    >>color1 += texture2D(texture, pos+dpos) * k;
    vec2 dposx = vec2(dx, 0.0);
    vec2 dposy = vec2(0.0, dy);
    vec4 gradx = texture2D(texture, pos+dpos+dposx) - texture2D(texture, pos+dpos-dposx);
    vec4 grady = texture2D(texture, pos+dpos+dposy) - texture2D(texture, pos+dpos-dposy);
    vec4 tmpColor = gradx*gradx + grady*grady;
    tmpColor = sqrt(tmpColor);
    tmpColor.a = texture2D(texture, pos+dpos).a;
    color1 += tmpColor * k;
""")

# Read image
im = vv.imread('astronaut.png')

# Show two times, the second will be sharpened
t = vv.imshow(im)

# Insert our part in the fragment shader program
t.shader.fragment.AddPart(SH_2F_EDGE)
if False: # Use this line to switch back:
    t.shader.fragment.RemovePart(SH_2F_EDGE)

# In case there are bugs in the code, it might be helpfull to see the code
# t.shader.fragment.ShowCode() # Shows the whole code
t.shader.fragment.ShowCode('edge') # Shows only our bit, with line numbers

# Run app
app = vv.use()
app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def close(fig):
    """ close(fig)
    
    Close a figure.
    
    fig can be a Figure object or an integer representing the id of the
    figure to close. Note that for the first case, you migh also call
    fig.Destroy().
    
    """
    
    if isinstance(fig, int):
        figDict = vv.BaseFigure._figures
        if fig in figDict:
            figDict[fig].Destroy()
        else:
            raise ValueError('A figure whith that id does not exist: '+str(fig))
    elif isinstance(fig, vv.BaseFigure):
        fig.Destroy()
    else:
        raise ValueError('Invalid argument for vv.functions.close')

if __name__ == '__main__':
    import time
    # Create figure
    fig = vv.figure(1)
    vv.processEvents(); vv.processEvents()
    time.sleep(1.0)
    # Close it
    vv.close(1) # Note: you can also pus the fig object



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module buttons

Implements a few button widgets.

"""

from visvis.core.events import BaseEvent, MouseEvent
from visvis.core.misc import Property
from visvis import Label


class PushButton(Label):
    """ PushButton(parent, text='', fontname=None)
    
    A button to click on. It is a label with an edgewidth of 1, in which
    text is horizontally aligned. Plus the hittest is put on by default.
    
    """
    
    def __init__(self, parent, *args, **kwargs):
        Label.__init__(self, parent, *args, **kwargs)
        
        # Get an edge
        self.edgeWidth = 1
        
        # Text is centered by default
        self.halign = 0
        
        # And changes appearance on mouse over
        self._isOver = False
        self.eventEnter.Bind(self._OnEnter)
        self.eventLeave.Bind(self._OnLeave)
        
        # Create new event and bind handlers to implement it
        self._eventPress = MouseEvent(self)
        self.eventMouseUp.Bind(self._OnPressDetectUp)
    
    
    def _OnEnter(self, event):
        self._isOver = True
        self.Draw()
    
    def _OnLeave(self, event):
        self._isOver = False
        self.Draw()
    
    def _GetBgcolorToDraw(self):
        """ _GetBgcolorToDraw()
        
        Can be overloaded to indicate mouse over in buttons.
        
        """
        clr = list(self._bgcolor)
        if self._isOver:
            clr = [c+0.05 for c in clr]
        return clr
    
    @property
    def eventPress(self):
        """ Fired when the mouse released over the button after a mouseDown.
        """
        return self._eventPress
    
    def _OnPressDetectUp(self, evt):
        if self._isOver:
            self._eventPress.Set(evt.absx, evt.absy, evt.button)
            self._eventPress.Fire()


class ToggleButton(PushButton):
    """ ToggleButton(parent, text='', fontname=None)
    
    Inherits from PushButton. This button can be set on and off by clicking
    on it, or by using it's property "state". The on state is indicated by
    making the edge thicker. The event eventStateChanged is fired when its
    state is changed.
    
    """
    
    def __init__(self, parent, *args, **kwargs):
        PushButton.__init__(self, parent, *args, **kwargs)
        
        # The state of the button, and an event to notify its change
        self._state = False
        self._eventStateChanged = BaseEvent(self)
        
        # Bind event
        self.eventMouseDown.Bind(self._OnDown)
    
    @Property # _Update will invoke a draw
    def state():
        """ A boolean expressing the state of the toggle button: on or off.
        """
        def fset(self, value):
            self._state = bool(value)
            self._Update()
        def fget(self):
            return self._state
        return locals()
    
    @property
    def eventStateChanged(self):
        """ Fires when the state of the button changes.
        """
        return self._eventStateChanged
    
    def _OnDown(self, event):
        self.state = not self.state
    
    def _Update(self, event=None):
        if self._state:
            self.edgeWidth = 2
        else:
            self.edgeWidth = 1
        #self.Draw() Setting edgeWidth will invoke redraw
        self._eventStateChanged.Fire()


class RadioButton(ToggleButton):
    """ RadioButton(parent, text='', fontname=None)
    
    Inherits from ToggleButton. If pressed upon, sets the state of all
    sibling RadioButton instances to False, and its own state to True.
    
    When this happens, all instances will fire eventStateChanged (after
    the states are set). So it's only necessary to bind to one of them
    to detect the selection of  another item.
    
    """
    
    def _OnDown(self, event):
        
        # Get me and all my siblings
        toggleButtons = []
        if self.parent:
            for sibling in self.parent.children:
                if isinstance(sibling, ToggleButton):
                    toggleButtons.append(sibling)
        
        # Disable all siblings (including self) and set own state
        for but in toggleButtons:
            but._state = False
        self._state = True
        
        # Update all so the events are fired
        for but in toggleButtons:
            but._Update()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np
import OpenGL.GL as gl

from visvis.processing.statistics import StatData
from visvis.core.misc import basestring


# todo: enable notch in boxplot

def boxplot(data1, data2=None, width=0.75, whiskers=1.5, axesAdjust=True, axes=None):
    """ boxplot(*args, width=0.75, whiskers=1.5, axesAdjust=True, axes=None)
    
    Create a box and whisker plot and returns a BoxPlot wobject that can
    be used to change the appearance of the boxes (such as color).
    
    If whiskers=='violin' creates a violin plot, which displays the
    kernel density estimate (kde) of each data.
    
    Usage
    -----
      * boxplot(data, ...) creates boxplots for the given list of data
      * boxplot(X, data, ...) also supply x-coordinates for each data
    
    Arguments
    ---------
    X : iterable (optional)
        Specify x position of the boxes.
    data : list
        List of data, where each data is a sequence (something that can be
        passed to numpy.array()).
    width : scalar
        The width of the boxes.
    whiskers : scalar or string
        How to draw the whiskers. If a scalar is given, it defines the
        length of the whiskers as a function of the IQR. In this case any
        points lying beyond the whiskers are drawn as outliers.
        If 'minmax', the whiskers simply extend to the maximal data range.
        If 'std', the whiskers indicate the mean +/- the standard deviation.
        If 'violin', a violin plot is drawn, which shows the probability
        density function completely.
    axesAdjust : bool
        If True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet,
        it is set to False.
    axes : Axes instance
        Display the bars in the given axes, or the current axes if not given.
    
    """
    
    # Get axes
    if axes is None:
        axes = vv.gca()
    
    # Pre check
    if data2 is None:
        data_list = data1
    else:
        data_list = data2
    if not isinstance(data_list, (tuple,list)):
        raise ValueError('Data should be given as a list.')
    #
    if data2 is None:
        xx = list(range(len(data_list)))
    else:
        xx = [float(x) for x in data1]
        if len(data_list) != len(xx):
            raise ValueError('Positions do not match length of data.')
    
    # Remove data where x position is invalid
    # At the same time sort data by x position
    validIndices, = np.where(np.isfinite(xx))
    validIndices = sorted(validIndices, key=lambda i:xx[i])
    xx = [xx[i] for i in validIndices]
    data_list = [data_list[i] for i in validIndices]
    
    # Create boxes and boxplot object
    boxes = [BoxPlotBox(d, width, whiskers) for d in data_list]
    bp = BoxPlot(axes, xx, boxes)
    
    # Adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = True
        axes.cameraType = '2d'
        axes.SetLimits()
    
    # Done
    axes.Draw()
    return bp


class BoxPlotBox(object):
    """ BoxPlotBox
    
    Represents a block in a boxplot. Used for storing information
    such as position of box and whiskers etc.
    
    """
    def __init__(self, data, width, whiskers):
        
        # Get stats of data
        self._stats = StatData(data)
        
        # Init width
        self._width = width
        
        # Init whisker style
        self._whiskers = whiskers
        
        # Init line style
        self._lc = (0,0,0)
        self._lw = 1
        
        # Calculate now
        self.calculate()
    
    
    def SetWidth(self, w):
        self._width = w
        self.calculate()
    
    
    def SetWhiskers(self, whiskers):
        
        # Lowercase and check
        
        if isinstance(whiskers, basestring):
            whiskers = whiskers.lower()
            if whiskers == 'default':
                whiskers = 1.5
            elif whiskers not in ['minmax', 'std', 'violin']:
                raise ValueError('Invalid whiskers style')
        elif isinstance(whiskers, (float, int)):
            whiskers = float(whiskers)
        else:
            raise ValueError('Invalid whiskers style')
        
        # Set
        self._whiskers = whiskers
        self.calculate()
    
    
    def calculate(self):
        """ calculate()
        
        Calculate the stats, and storing them such that they can
        be drawn easily.
        
        """
        
        # Init limts
        self._limits = vv.Range(self._stats.dmin, self._stats.dmax)
        
        # Calculate more?
        if isinstance(self._whiskers, float):
            self.calculate_outliers()
        elif self._whiskers == 'violin':
            self.calculate_violin()
        else:
            pass # we have all the info we need
    
    
    def calculate_outliers(self):
        
        # Get stats and data
        stats = self._stats
        data = stats._data
        
        # Set border
        whiskerWidth = 1.5
        if isinstance(self._whiskers, float):
            whiskerWidth = self._whiskers
        
        # Get indices of points beyond whiskers
        w1 = stats.Q1 - stats.IQR * whiskerWidth
        w2 = stats.Q3 + stats.IQR * whiskerWidth
        I1, = np.where(data < w1)
        I2, = np.where(data > w2)
        
        # Get points for whiskers
        if I1.size:
            self._wmin = data[ I1[-1]+1 ]
        else:
            self._wmin = data[0]
        if I2.size:
            self._wmax = data[ I2[0]-1 ]
        else:
            self._wmax = data[-1]
        
        # Get outlier points
        Iall = np.concatenate([I1, I2])
        self._outliers = data[ Iall ]
    
    
    def calculate_violin(self):
        
        # Get stats
        stats = self._stats
        
        # Get kernel density estimate
        nbins = stats.best_number_of_bins(8, 128)
        centers, values = stats.kde(nbins)
        
        # Normalize values
        values = values * (0.5 * self._width / values.max())
        
        # Create array with locations
        n = values.size
        points = np.zeros((n*2+1,3), np.float32)
        points[:n,0] = values
        points[:n,1] = centers
        points[n:2*n,0] = -np.flipud(values)
        points[n:2*n,1] = np.flipud(centers)
        points[2*n,0] = values[0]
        points[2*n,1] = centers[0]
        #
        self._points = points
        
        # Update limits
        self._limits = vv.Range(centers[0], centers[-1])
    
    
    def Draw(self, x_offset):
        
        # Prepare color and line width
        clr = self._lc
        gl.glColor3f(clr[0], clr[1], clr[2])
        gl.glLineWidth(self._lw)
        
        # Set line smoothing
        if self._lw == int(self._lw):
            gl.glDisable(gl.GL_LINE_SMOOTH)
        else:
            gl.glEnable(gl.GL_LINE_SMOOTH)
        
        # Draw
        if self._whiskers == 'violin':
            self.DrawViolin(x_offset)
        else:
            self.DrawBox(x_offset)
        
        # Reset
        gl.glEnable(gl.GL_LINE_SMOOTH)
    
    
    def DrawBox(self, x_offset):
        
        # Get data
        stats = self._stats
        
        # Determine whisker position
        if isinstance(self._whiskers, float):
            wmin, wmax = self._wmin, self._wmax
        elif self._whiskers == 'minmax':
            wmin, wmax = stats.dmin, stats.dmax
        elif self._whiskers == 'std':
            wmin, wmax = stats.mean - stats.std, stats.mean + stats.std
        
        
        # Relative width
        w1 = self._width * 0.5
        w2 = self._width * 0.125
        
        # Draw box
        gl.glBegin(gl.GL_LINE_LOOP)
        gl.glVertex2f(x_offset-w1, stats.Q1)
        gl.glVertex2f(x_offset+w1, stats.Q1)
        gl.glVertex2f(x_offset+w1, stats.Q3)
        gl.glVertex2f(x_offset-w1, stats.Q3)
        gl.glEnd()
        
        # Draw mean, wisker lines, and wiskers
        gl.glBegin(gl.GL_LINES)
        gl.glVertex2f(x_offset-w1, stats.Q2)
        gl.glVertex2f(x_offset+w1, stats.Q2)
        #
        gl.glVertex2f(x_offset, stats.Q1)
        gl.glVertex2f(x_offset, wmin)
        gl.glVertex2f(x_offset, stats.Q3)
        gl.glVertex2f(x_offset, wmax)
        #
        gl.glVertex2f(x_offset-w2, wmin)
        gl.glVertex2f(x_offset+w2, wmin)
        gl.glVertex2f(x_offset-w2, wmax)
        gl.glVertex2f(x_offset+w2, wmax)
        gl.glEnd()
        
        # Draw outliers?
        if isinstance(self._whiskers, float):
            # Group outliers
            outliers = []
            for p in self._outliers:
                if outliers and outliers[0][-1] == p:
                    outliers[0].append(p)
                else:
                    outliers.append([p])
            # Draw outliers
            w3 = self._width * 0.125*0.5
            gl.glPointSize(5)
            gl.glEnable(gl.GL_POINT_SMOOTH)
            gl.glBegin(gl.GL_POINTS)
            for outlierGroup in outliers:
                offset = - (len(outlierGroup)-1) * w3 * 0.5
                offset = max(offset, -w1)
                for i in range(len(outlierGroup)):
                    p = outlierGroup[i]
                    gl.glVertex2f(x_offset+offset+w3*i, p)
            gl.glEnd()
    
    
    def DrawViolin(self, x_offset):
        
        # Get stats
        stats = self._stats
        
        # Smooth lines
        gl.glEnable(gl.GL_LINE_SMOOTH)
        
        # Translate points
        points = self._points.copy()
        points[:,0] += x_offset
        
        # Draw outer lines
        gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
        gl.glVertexPointerf(points)
        gl.glDrawArrays(gl.GL_LINE_STRIP, 0, points.shape[0])
        gl.glFlush()
        gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
        
        # Draw mean p25, p50 and p75
        w2 = self._width * 0.25
        gl.glBegin(gl.GL_LINES)
        gl.glVertex2f(x_offset-w2, stats.Q2)
        gl.glVertex2f(x_offset+w2, stats.Q2)
        gl.glVertex2f(x_offset, stats.Q1)
        gl.glVertex2f(x_offset, stats.Q3)
        gl.glEnd()
    

class BoxPlot(vv.Wobject):
    """
    
    """
    def __init__(self, parent, xx, boxes):
        vv.Wobject.__init__(self, parent)
        self._xx = xx
        self._boxes = boxes
    
    
    def OnDraw(self):
        
        # Draw all boxes
        for i in range(len(self._boxes)):
            self._boxes[i].Draw(self._xx[i])
    
    
    def _GetLimits(self):
        
        # xlim and zlim are easy
        #x1, x2 = 0.5, len(self._boxes)-0.5
        x1 = self._xx[0] - self._boxes[0]._width * 1.05
        x2 = self._xx[-1] + self._boxes[-1]._width * 1.05
        z1, z2 = 0, 0.2
        
        # ylim is harder
        y1, y2 = 9999999999999, -99999999999999999
        for box in self._boxes:
            y1 = min(y1, box._stats.dmin)
            y2 = max(y2, box._stats.dmax)
        if not self._boxes:
            y1, y2 = 0,1
        
        # Done
        return vv.Wobject._GetLimits(self, x1, x2, y1, y2, z1, z2)
    
    
    @vv.misc.PropWithDraw
    def lc():
        """ Get/Set the line color of the boxes.
        """
        def fget(self):
            return self._boxes[0]._lc
        def fset(self, value):
            lc = vv.misc.getColor(value, 'setting line color')
            for box in self._boxes:
                box._lc = lc
        return locals()
    
    @vv.misc.PropWithDraw
    def lw():
        """ Get/Set the line width of the boxes.
        """
        def fget(self):
            return self._boxes[0]._lw
        def fset(self, value):
            for box in self._boxes:
                box._lw = float(value)
        return locals()
    
    
    @vv.misc.PropWithDraw
    def whiskers():
        """ Get/Set the style of the whiskers.
        """
        def fget(self):
            return self._boxes[0]._whiskers
        def fset(self, value):
            for box in self._boxes:
                box.SetWhiskers(value)
        return locals()


if __name__ == '__main__':
    # Create data
    d1 = np.random.normal(1, 4, (1000,1000))
    d2 = np.random.normal(2, 3, (20,))
    d3 = np.random.uniform(-1, 3, (100,))
    d4 = [1,2,1,2.0, 8, 2, 3, 1, 2, 2, 3, 2, 2.1, 8, 8, 8, 8, 8,  1.2, 1.3, 0, 0, 1.5, 2]
    # Show boxplot
    b = vv.boxplot((d1,d2,d3, d4), width=0.8, whiskers='violin')



# This file is part of VISVIS.
# Copyright (C) 2012 Almar Klein

import visvis as vv
import numpy as np


def getCardinalSplineCoefs(t, tension=0.0):
    
    # Init
    t = float(t)
    c = [0 for i in range(4)]
    tau  = 0.5*(1-tension)
    
    # Calc coefs
    c[0] = - tau * (   t**3 - 2*t**2 + t )
    c[3] =   tau * (   t**3 -   t**2     )
    c[1] =           2*t**3 - 3*t**2 + 1  - c[3]
    c[2] = -         2*t**3 + 3*t**2      - c[0]
    
    # Done
    return c


def screenshot(filename, ob=None, sf=2, bg=None, format=None, tension=-0.25):
    """ screenshot(filename, ob=None sf=2, bg=None, format=None)
    
    Make a screenshot and store it to a file, using cubic interpolation
    to increase the resolution (and quality) of the image.
    
    Parameters
    ----------
    filename : string
        The name of the file to store the screenshot to. If filename is None,
        the interpolated image is returned as a numpy array.
    ob : Axes, AxesContainer, or Figure
        The object to take the screenshot of. The AxesContainer can be
        obtained using vv.gca().parent. It can be usefull to take a
        screeshot of an axes including thickmarks and labels.
    sf : integer
        The scale factor. The image is increased in size with this factor,
        using a high quality interpolation method. A factor of 2 or 3
        is recommended; the image quality does not improve with higher
        factors. If using a sf larger than 1, the image is best saved in
        the jpg format.
    bg : 3-element tuple or char
        The color of the background. If bg is given, ob.bgcolor is set to
        bg before the frame is captured.
    format : string
        The format for the screenshot to be saved in. If not given, the
        format is deduced from the filename.
    
    Notes
    -----
    Uses vv.getframe(ob) to obtain the image in the figure or axes.
    That image is interpolated with the given scale factor (sf) using
    bicubic interpolation. Then  vv.imwrite(filename, ..) is used to
    store the resulting image to a file.
    
    Rationale
    ---------
    We'd prefer storing screenshots of plots as vector (eps) images, but
    the nature of OpenGl prevents this. By applying high quality
    interpolation (using a cardinal spline), the resolution can be increased,
    thereby significantly improving the visibility/smoothness for lines
    and fonts. Use this to produce publication quality snapshots of your
    plots.
    
    """
    
    # The tension controls the
    # responsivenes of the filter. The more negative, the more overshoot,
    # but the more it is capable to make for example font glyphs smooth.
    # If tension is 0, the interpolator is a Catmull-Rom spline.
    
    # Scale must be integer
    s = int(sf)
    
    # Object given?
    if ob is None:
        ob = vv.gcf()
    
    # Get figure
    fig = ob
    if not hasattr(ob, 'DrawNow'):
        fig = ob.GetFigure()
    
    # Get object to set background of
    bgob = ob
    
    # Set background
    if bg and fig:
        bgOld = bgob.bgcolor
        bgob.bgcolor = bg
        fig.DrawNow()
    
    # Obtain image
    im1 = vv.getframe(ob)
    shape1 = im1.shape
    
    # Return background
    if bg and fig:
        bgob.bgcolor = bgOld
        fig.Draw()
    
    # Pad original image, so we have no trouble at the edges
    shape2 = shape1[0]+2, shape1[1]+2, 3
    im2 = np.zeros(shape2, dtype=np.float32) # Also make float
    im2[1:-1,1:-1,:] = im1
    im2[0,:,:] = im2[1,:,:]
    im2[-1,:,:] = im2[-2,:,:]
    im2[:,0,:] = im2[:,1,:]
    im2[:,-1,:] = im2[:,-2,:]
    
    # Create empty new image. It is sized by the scaleFactor,
    # but the last row is not.
    shape3 = (shape1[0]-1)*s+1, (shape1[1]-1)*s+1, 3
    im3 = np.zeros(shape3, dtype=np.float32)
    
    # Fill in values!
    for dy in range(s+1):
        for dx in range(s+1):
            
            # Get interpolation fraction and coefs
            ty = float(dy)/s
            tx = float(dx)/s
            cy = getCardinalSplineCoefs(ty, tension)
            cx = getCardinalSplineCoefs(tx, tension)
            
            # Create tmp image to which we add the contributions
            # Note that this image is 1 pixel smaller in each dimension.
            # The last pixel is filled because dy and dx iterate INCLUDING s.
            shapeTmp = shape1[0]-1, shape1[1]-1, 3
            imTmp = np.zeros(shapeTmp, dtype=np.float32)
            
            # Collect all 16 neighbours and weight them apropriately
            for iy in range(4):
                for ix in range(4):
                    
                    # Get weight
                    w = cy[iy]*cx[ix]
                    if w==0:
                        continue
                    
                    # Get slice. Note that we start at 0,1,2,3 rather than
                    # -1,0,1,2, because we padded the image.
                    D = {0:-3,1:-2,2:-1,3:None}
                    slicey = slice(iy, D[iy])
                    slicex = slice(ix, D[ix])
                    
                    # Get contribution and add to temp image
                    imTmp += w * im2[slicey, slicex, :]
            
            # Store contributions
            D = [-1 for tmp in range(s)]; D.append(None)
            slicey = slice(dy,D[dy],s)
            slicex = slice(dx,D[dx],s)
            im3[slicey, slicex, :] = imTmp
    
    
    # Correct for overshoot
    im3[im3>1]=1
    im3[im3<0]=0
    
    # Store image to file
    if filename is not None:
        vv.imwrite(filename, im3, format)
    else:
        return im3


if __name__ == '__main__':
    # Make plot
    vv.plot([1,2,3,1,4,2,3], ms='.')
    # Take screenshot, twice enlarged, on a white background
    vv.screenshot('screenshot.jpg', vv.gcf(), sf=2, bg='w')



""" Module graph

Defines functionality to manage, draw and store graphs. A graph is
represented using an adjacency list; the root of this module is the
Graph class, which represents a list of Node objects, which are in turn
connected by Edge instances.

The objects can be overloaded to provide application specific functionality,
if required.

This module also defines functions and classes to compare graphs.

Note that there is also a package called "graphine". The main difference
of visvis.utils.graph is that it is based on visvis.Point's and the nodes
explicitly have a location in a 2D or 3D space.

Example
-------

# Create graph with nodes
g = Graph()
n1 = g.AppendNode(3,4)
n2 = g.AppendNode(5,7)
n3 = g.AppendNode(3,8)
n4 = g.AppendNode(2,5)
n5 = g.AppendNode(4,6)

# Define connections
g.CreateEdge(n1, n2)
g.CreateEdge(n2, n3)
g.CreateEdge(n1, n4)
g.CreateEdge(n2, n4)
g.CreateEdge(n1, n5)

# Draw it
vv.clf()
a = vv.gca()
g.Draw(lc='b')
a.camera = 2
a.SetLimits()

# Get some info
print(g.GetEdges())

"""

import numpy as np
import visvis as vv
from visvis import Point, Pointset
from visvis import ssdf


class Node(Point):
    """ Node(x,y,z,...)
    A Node object represents a point in 2D or 3D space, i.e. a vertex.
    It has a list of references to edge objects. Via these
    edges, the neighbouring node objects can be obtained.
    This class inherits the Point object.
    Use Graph.AppendNode() to create nodes.
    """
    
    def __init__(self, *args, **kwargs):
        Point.__init__(self, *args, **kwargs)
        self._edges = []
    
    
    def __hash__(self):
        return id(self)
    
    def GetEdge(self, otherNode):
        """ GetEdge(self, otherNode)
        Gives the edge from this node to the given node. If there
        is no edge between these nodes, None is returned.
        """
        for c in self._edges:
            if otherNode is c.GetOtherEnd(self):
                return c
        else:
            return None
    
    
    def GetNeighbours(self):
        """ GetNeighbours()
        Get a list of the nodes that this node is adjecent to.
        """
        return [c.GetOtherEnd(self) for c in self._edges]
    
    
    @property
    def degree(self):
        """ The number of neighbours of this node. """
        return len(self._edges)
    
    
    def GetProps(self, index):
        """ GetProps(index)
        Get a list of the property at the given index asociated with the
        edges to the neighboring nodes, matching with what is returned
        by GetNeighbours.
        """
        return [c.props[index] for c in self._edges]
    
    
    def __str__(self):
        """ Make string representation.
        """
        s = Point.__str__(self)
        s.replace('point', 'node')

class Edge:
    """ Edge(node1, node2, *props)
    The base edge class. An edge consists of the references
    of the two nodes it connects and a list of properties asociated with
    the edge.
    Use Graph.CreateEdge to create edges.
    """
    
    def __init__(self, p1, p2, *props):
        if not isinstance(p1, Node) or not isinstance(p2, Node):
            raise ValueError("First two arguments of an edge should be nodes.")
        
        # The two end points
        self.end1 = p1
        self.end2 = p2
        # Store properties
        self.props = [prop for prop in props]
        
        # The indices of the end nodes, only make sense right after calling
        # Graph.GetEdges()
        self._i1 = self._i2 = -1
        
        # A varible to bookkeep whether the edge is connected or not
        self._connected = False
    
    
    def IsEnd(self, end):
        """ IsEnd(self, end)
        Returns True if the given node is a valid end. """
        return (end is self.end1) or (end is self.end2)
    
    
    def GetOtherEnd(self, end):
        """ GetOtherEnd(self, end)
        Given one node, get the node representing the other end of
        the edge. """
        if not self.IsEnd(end):
            raise ValueError("Given end is not a valid end for this edge.")
        if self.end1 is end:
            return self.end2
        else:
            return self.end1
    
    
    def Connect(self):
        """ Connect(self)
        Connect the edge, registering it with both end nodes.
        """
        # make sure not to have the same edge twice
        self.Disconnect()
        # append edge
        self.end1._edges.append(self)
        self.end2._edges.append(self)
        self._connected = True
    
    
    def Disconnect(self):
        """ Disconnect(self)
        Disconnect the edge, unregistering it with the end nodes.
        """
        while self in self.end1._edges:
            self.end1._edges.remove(self)
        while self in self.end2._edges:
            self.end2._edges.remove(self)
        self._connected = False
   


class Graph(list):
    """ Graph()
    
    A Graph represents a collection of nodes that are connected by edges.
    The nodes can be either 2D or 3D points.
    
    Use AppendNode to create nodes and CreateEdge to connect them.
    
    Notes
    -----
    This class cannot inherit from Pointset, because we need to store
    the Node instances to keep track of which nodes are connected by
    which.
    
    Inherits from list, newly implemented methods can be recocnized
    because they start with a capital letter. The use of these methods
    is preferred over the lower case versions.
    
    """
    
    def __init__(self):
        
        # for drawing
        self._lines = []
        self._activeLines = []
    
    
    def __repr__(self):
        tmp =  '<Graph with %i nodes and %i edges at %s>'
        return tmp % (len(self), self.CountEdges(), hex(id(self)))
    
    
    def __to_ssdf__(self):
        return self.Pack()
    
    @classmethod
    def __from_ssdf__(self, s):
        g = Graph()
        g.Unpack(s)
        return g
    
    
    def Pack(self):
        """ Pack()
        Pack the contents in an ssdf struct, such that it can be stored.
        """
        
        # If nodelist is empty, ndim defaults to two
        ndim = 2
        
        # Get a list of all edges
        cc = self.GetEdges()
        
        # Check whether the nodes are homogenous, otherwise we cannot store
        if len(self):
            ndim = self[0].ndim
            for node in self:
                if not node.ndim == ndim:
                    raise ValueError('All nodes should have the same dimension.')
        
        # Init struct
        struct = ssdf.new()
        
        # Create array of nodes
        pp = Pointset(ndim)
        for node in self:
            pp.append(node)
        struct.nodes = pp.data
        
        # Create the edges
        tmp = np.zeros((len(cc), 2), dtype=np.uint32)
        for i in range(len(cc)):
            c = cc[i]
            tmp[i,0] = c._i1
            tmp[i,1] = c._i2
        struct.edges = tmp
        
        
        # Store the properties of the edges. The propRefs array
        # Does contain redundant data, but it can be stored efficiently
        # because the compression handles that...
        allProps = b''
        propRefs = np.zeros((len(cc), 2), dtype=np.uint32)
        for i in range(len(cc)):
            tmp = serializeProps(cc[i].props)
            propRefs[i, 0] = len(allProps)
            propRefs[i, 1] = len(allProps) + len(tmp)
            allProps += tmp
        struct.edgePropRefs = propRefs
        if allProps:
            struct.edgeProps = np.frombuffer( allProps, dtype=np.uint8)
        else:
            struct.edgeProps = np.zeros( (0,), dtype=np.uint8)
        
        # done
        return struct
    
    
    def Unpack(self, struct):
        """ Unpack(struct)
        Load the contents from an ssdf struct.
        This removes the old nodes and edges.
        """
        
        # clear first
        self.Clear()
        
        # First create nodes
        pp = Pointset(struct.nodes)
        for p in pp:
            self.AppendNode(p)
        
        # Make backward compatible
        if hasattr(struct, 'connections'):
            struct.edges = struct.connections
            struct.edgePropRefs = struct.connectionPropRefs
            struct.edgeProps = struct.connectionProps
        
        # Create the edges
        for i in range( struct.edges.shape[0] ):
            # Get serialized property and convert it
            i1 = struct.edgePropRefs[i,0]
            i2 = struct.edgePropRefs[i,1]
            tmp = struct.edgeProps[i1:i2].tostring()
            props = deserializeProps(tmp)
            # Get which nodes belong to it and create edge object
            i1 = int(struct.edges[i,0])
            i2 = int(struct.edges[i,1])
            self.CreateEdge(i1, i2, *props)
    
    
    def Clear(self):
        """ Clear()
        Clear all nodes and edges.
        """
        self[:]=[]
    
    
    def ClearEdges(self):
        """ ClearEdges()
        Clear the edges only.
        """
        for node in self:
            node._edges[:] = []
    
    
    def Copy(self):
        """ Copy()
        Create a full copy of the object (using pack/unpack).
        """
        tmp = self.Pack()
        new = Graph()
        new.Unpack(tmp)
        return new
    
    
    def CountEdges(self):
        """ CountEdges()
        Count the number of edges. """
        count = 0
        for node in self:
            count += len(node._edges)
        return count/2 # because we count each edge twice
    
    
    def GetEdges(self):
        """ GetEdges()
        Get a list of all edges in the graph.
        This is obtained by collecting the edges from all nodes.
        This also sets the _i1 and _i2 properties of all edge
        objects: the index of the two nodes it connects. Note that
        these values become invalid when a node is popped or inserted.
        """
        # Init set (so that each edge is present only once)
        cc = set()
        
        # Check each node
        for i in range(len(self)):
            p = self[i]
            
            # Check each edge
            for c in p._edges:
                # Add if not already added
                cc.add(c)
                # Add reference to the index of this node.
                if c.end1 is p:
                    c._i1 = i
                else:
                    c._i2 = i
        
        # Make list and sort so the order is deterministic
        ccl = list(cc)
        ccl.sort(key=edgeHash)
        
        # Done
        return ccl
    
    
    #def CollectGroups(self, asGraphs=False):
    def CollectGroups(self):
        """ CollectGroups()
        
        Collect all groups of the graph.
        
        """
        
        # Init empty list of groups, each group being a set
        groups = []
        
        # Init search heap
        nodes2search = set([node for node in self])
        
        # Collect group
        while nodes2search:
            # Initialize a new group with one node
            group1 = set() # initial
            group2 = set() # final
            group1.add(nodes2search.pop())
            
            # Search the whole group
            while group1:
                # Take a note from group 1
                node = group1.pop()
                # Make official: put in group2, remove from nodes2search
                group2.add(node)
                nodes2search.discard(node)
                # Pul all its neighbours in group 1, if not alread in group 2
                for neighbour in [c.GetOtherEnd(node) for c in node._edges]:
                    if neighbour not in group2:
                        group1.add(neighbour)
            
            # Add group
            groups.append(group2)
        
        # todo: Enable returning the groups as graph objects?
        
        return groups

    
    def AppendNode(self, *p):
        """ AppendNode(p)
        Append a node (if a point is given, it is converted to a node).
        Overload this to append nodes of a specific class. Returns the
        appended instance.
        """
        if len(p) == 1:
            p = p[0]
        
        if isinstance(p, Node):
            pass
        elif isinstance(p, Point):
            p = Node(p)
        else:
            #raise ValueError("Only Node objects should be appended.")
            # Try to accept it as a point
            p = Node(p)
        # Append and return
        self.append(p)
        return p
    
    
    def _CheckNodes(self, p1, p2):
        """ _CheckNodes(p1,p2)
        Check the given nodes. If they are indices the corresponding
        node objects are returned. If they are node objects, it's
        checked whether they are in the list.
        """
        # Convert indices to nodes, or check if it's a valid node
        if isinstance(p1, int):
            p1 = self[p1]
        else:
            if p1 not in self:
                raise ValueError("First given node not in list!")
        if isinstance(p2, int):
            p2 = self[p2]
        else:
            if p2 not in self:
                raise ValueError("Second given node not in list!")
        
        return p1, p2
    
    
    def CreateEdge(self, p1, p2, *props):
        """ CreateEdge(self, p1, p2, *props)
        Creates an edge instance between the two given nodes.
        The edge replaces any existing edge.
        """
        
        # Check nodes
        p1, p2 = self._CheckNodes(p1, p2)
        
        # Get new and old edge
        cnew = Edge(p1, p2, *props)
        cold = p1.GetEdge(p2) # can be None
        
        # Disconnect old one if necesary. Connect new one
        if cold is not None:
            cold.Disconnect()
        cnew.Connect()
        
        # return edge
        return cnew
    
    
    def Remove(self, node):
        """ Remove(node)
        Remove the node (which can also be given using its index,
        disconnecting all its edges.
        """
        if isinstance(node, int):
            node = self[node]
        for c in [c for c in node._edges]:
            c.Disconnect()
        self.remove(node)
    
    
    def Draw(self, mc='g', lc='y', mw=7, lw=0.6, alpha=0.5, axes=None):
        """ Draw(mc='g', lc='y', mw=7, lw=0.6, alpha=0.5, axes=None)
        Draw nodes and edges.
        """
        
        # We can only draw if we have any nodes
        if not len(self):
            return
        
        # Make sure there are elements in the lines list
        while len(self._lines)<2:
            self._lines.append(None)
        
        # Build node list
        if mc and mw:
            pp = Pointset(self[0].ndim)
            for p in self:
                pp.append(p)
            # Draw nodes, reuse if possible!
            l_node = self._lines[0]
            if l_node and len(l_node._points) == len(pp):
                l_node.SetPoints(pp)
            elif l_node:
                l_node.Destroy()
                l_node = None
            if l_node is None:
                l_node = vv.plot(pp, ls='', ms='o', mc=mc, mw=mw,
                    axesAdjust=0, axes=axes, alpha=alpha)
                self._lines[0] = l_node
        
        # For simplicity, always redraw edges
        if self._lines[1] is not None:
            self._lines[1].Destroy()
        
        # Build edge list
        if lc and lw:
            cc = self.GetEdges()
            # Draw edges
            pp = Pointset(self[0].ndim)
            for c in cc:
                pp.append(c.end1); pp.append(c.end2)
            tmp = vv.plot(pp, ms='', ls='+', lc=lc, lw=lw,
                axesAdjust=0, axes=axes, alpha=alpha)
            self._lines[1] = tmp
    

## Helper classes and functions


def edgeHash(x):
    """ edgeHash(edge)
    Produces a hash for an edge that can be used in sorting.
    requires the edges to be "marked with node indices". Will use
    the first property (if available).
    """
    if x.props:
        return '%0.30f' % x.props[0] + '_' + str(x._i1) + '_' + str(x._i2)
    else:
        return '_' + str(x._i1) + '_' + str(x._i2)
    
def serializeProps(props):
    """ serializeProps(props)
    Encodes the properties of an edge in a bytes string
    for efficient storing.
    """
    
    # Init data bits
    databits = b''
    
    # Create bits of data
    for prop in props:
        if isinstance(prop, float):
            tmp = np.array(prop, dtype=np.float32)
            databits += b'\x00'
            databits += tmp.tostring()
        elif isinstance(prop, int):
            tmp = np.array(prop, dtype=np.int32)
            databits += b'\x01'
            databits += tmp.tostring()
        elif isinstance(prop, Pointset):
            databits += b'\x09'
            databits += np.array(len(prop), dtype=np.int32).tostring()
            databits += chr(prop.ndim).encode('utf-8')
            databits += prop.data.tostring()
        else:
            print('Warning: do not know how to store %s' % repr(prop))
    
    # Return
    return databits


def deserializeProps(bb):
    """ deserializeProps(bb)
    Set the properties by decoding them from a bytes string.
    """
    
    # Init list of properties
    props = []
    
    # Decode the string
    i = 0
    while i < len(bb):
        if bb[i] == b'\x00'[0]:
            # float
            tmp = np.frombuffer( bb[i+1:i+5], dtype=np.float32 )
            props.append( float(tmp) )
            i += 5
        elif bb[i] == b'\x01'[0]:
            # int
            tmp = np.frombuffer( bb[i+1:i+5], dtype=np.int32 )
            props.append( int(tmp) )
            i += 5
        elif bb[i] == b'\x09'[0]:
            n = np.frombuffer( bb[i+1:i+5], dtype=np.int32 )
            ndim = bb[i+5]
            if not isinstance(ndim, int): ndim = ord(ndim)
            nbytes = n*ndim*4
            tmp = np.frombuffer( bb[i+6:i+6+nbytes], dtype=np.float32 )
            tmp.shape = n, ndim
            props.append( Pointset(tmp) )
            i += 6 + nbytes
    
    # Return properties
    return props


## Functions and classes to compare graphs


class MatchingScore:
    """ MatchingScore(TP, FN, FP)
    
    Class to store a matching score using three indicaters
    represented by the matched (true-positive), missed (false-negative),
    and wrong (false-positive) amounts.
    
    The value (.val) is calculated as TP/(TP+FP+FN). Which is the
    intersection devided by the union of the two graphs. Let D=FP+FN be
    the "edit distance" between the two graphs. Then a sensible score
    measure that would scale between 0 and 1 would be 1-D/(D+TP). It can
    be shown that this is the same as TP/(TP+D).
    
    Supports equating and smaller/larger than operators (and thus sorting).
    Also supports combining matching scores by simply adding them, and
    un-combining by subtracting them.
    """
    
    def __init__(self, TP, FN, FP):
        # Calculate score and set
        if TP==0 and FP==0 and FN==0:
            val = -1 # Null matching score
        else:
            # How I had it first. Tannimoto Coefficient TC    DSC = 2*TC/(TC+1)
            #val = float(TP) / (TP+FP+FN)
            # Dice Similarity Coefficient (DSC)
            val = float(2*TP) / (2*TP + FP + FN)
        
        self.val = val
        # Store other attributes
        self.nmatch = TP
        self.nmiss = FN
        self.nwrong = FP
    
    def __lt__(self, other):
        if isinstance(other, MatchingScore): return self.val < other.val
        else: return self.val < other
    
    def __gt__(self, other):
        if isinstance(other, MatchingScore): return self.val > other.val
        else: return self.val > other
    
    def __eq__(self, other):
        if isinstance(other, MatchingScore): return self.val == other.val
        else: return self.val == other
    
    def __ne__(self, other):
        if isinstance(other, MatchingScore): return self.val != other.val
        else: return self.val != other
    
    def __repr__(self):
        formatstr = '<MatchingScore %1.2f with %i/%i/%i>'
        return formatstr % (self.val, self.nmatch, self.nmiss, self.nwrong)
    
    @property
    def value(self):
        return self.val
    
    @property
    def TP(self):
        return self.nmatch
    
    @property
    def FN(self):
        return self.nmiss
    
    @property
    def FP(self):
        return self.nwrong
    
    def toTuple(self):
        """ toTuple()
        return (self.TP, self.FN, self.FP)."""
        return (self.nmatch, self.nmiss, self.nwrong)
    
    
    def __add__(self, other):
        t = MatchingScore(self.TP+other.TP, self.FN+other.FN, self.FP+other.FP)
        return t
    
    
    def __sub__(self, other):
        t = MatchingScore(self.TP-other.TP, self.FN-other.FN, self.FP-other.FP)
        return t


def compareGraphs(graph1, graph2, maxDist):
    """ compareGraphs(graph1, graph2, maxDist)
    Compare two graphs to produce a matching score. Returns a MatchingScore
    instance.
    
    Matching and not-matching edges are counted to obtain a matching
    score. nodes should be closer than maxDist to be considered 'at the
    same location'.
    """
    
    # Check graphs
    if not graph1 or not graph2:
        print('Warning: one of the graphs to compare is empty.')
        return MatchingScore(0,1,0)
        #raise ValueError('One of the graphs to compare is empty.')
    
    # Create pointsets of the nodes
    pp1 = Pointset(3)
    for node in graph1:
        pp1.append(node)
    pp2 = Pointset(3)
    for node in graph2:
        pp2.append(node)
    
    # Match the nodes of graph1 to graph2
    for node in graph1:
        dists = node.distance(pp2)
        i, = np.where(dists==dists.min())
        node.match = None
        if len(i):
            i = int(i[0])
            dist = float(dists[i])
            if dist < maxDist:
                node.match = graph2[i]
#         if not hasattr(node, 'dontCare'):
#             node.dontCare = False
    
    # Match the nodes of graph2 to graph1
    for node in graph2:
        dists = node.distance(pp1)
        i, = np.where(dists==dists.min())
        node.match = None
        if len(i):
            i = int(i[0])
            dist = float(dists[i])
            if dist < maxDist:
                node.match = graph1[i]
#         if not hasattr(node, 'dontCare'):
#             node.dontCare = False
    
    # Init amounts
    nmatch1, nmatch2, nmiss, nwrong = 0,0,0,0
    
    # Count matches and wrongs
    for c in graph1.GetEdges():
        end1 = c.end1.match
        end2 = c.end2.match
        if end1 and end2:
            if end1 in end2.GetNeighbours():
                nmatch1 += 1
                continue
#         elif c.end1.dontCare or c.end2.dontCare:
#             continue
        nwrong += 1
    
    # todo: use dontCare 'beleid' or not?
    
    # Count matches and misses
    for c in graph2.GetEdges():
        end1 = c.end1.match
        end2 = c.end2.match
        if end1 and end2:
            if end1 in end2.GetNeighbours():
                nmatch2 += 1
                continue
#         elif c.end1.dontCare or c.end2.dontCare:
#             continue
        nmiss += 1
    
    # Compose score
    return MatchingScore(nmatch1, nmiss, nwrong)


def compareGraphsVisually(graph1, graph2, fig=None):
    """ compareGraphsVisually(graph1, graph2, fig=None)
    Show the two graphs together in a figure. Matched nodes are
    indicated by lines between them.
    """
    
    # Get figure
    if isinstance(fig,int):
        fig = vv.figure(fig)
    elif fig is None:
        fig = vv.figure()
    
    # Prepare figure and axes
    fig.Clear()
    a = vv.gca()
    a.cameraType = '3d'; a.daspectAuto = False
    
    # Draw both graphs
    graph1.Draw(lc='b', mc='b')
    graph2.Draw(lc='r', mc='r')
    
    # Set the limits
    a.SetLimits()
    
    # Make a line from the edges
    pp = Pointset(3)
    for node in graph1:
        if hasattr(node, 'match') and node.match is not None:
            pp.append(node); pp.append(node.match)
    
    # Plot edges
    vv.plot(pp, lc='g', ls='+')


if __name__ == '__main__':
    # Example
    
    # Create graph with nodes
    g = Graph()
    n1 = g.AppendNode(3,4)
    n2 = g.AppendNode(5,7)
    n3 = g.AppendNode(3,8)
    n4 = g.AppendNode(2,5)
    n5 = g.AppendNode(4,6)
    
    # Define connections
    g.CreateEdge(n1, n2)
    g.CreateEdge(n2, n3)
    g.CreateEdge(n1, n4)
    g.CreateEdge(n2, n4)
    g.CreateEdge(n1, n5)
    
    # Draw it
    vv.clf()
    a = vv.gca()
    g.Draw(lc='b')
    a.camera = 2
    a.SetLimits()
    
    # Get some info
    print(g.GetEdges())



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np
from visvis.utils.pypoints import Point, Pointset


def solidRing(translation=None, scaling=None, direction=None, rotation=None,
                thickness=0.25, N=16, M=16, axesAdjust=True, axes=None):
    """ solidRing(translation=None, scaling=None, direction=None, rotation=None,
                thickness=0.25, N=16, M=16, axesAdjust=True, axes=None)
    
    Creates a solid ring with quad faces oriented at the origin.
    Returns an OrientableMesh instance.
    
    Parameters
    ----------
    Note that translation, scaling, and direction can also be given
    using a Point instance.
    translation : (dx, dy, dz), optional
        The translation in world units of the created world object.
    scaling: (sx, sy, sz), optional
        The scaling in world units of the created world object.
    direction: (nx, ny, nz), optional
        Normal vector that indicates the direction of the created world object.
    rotation: scalar, optional
        The anle (in degrees) to rotate the created world object around its
        direction vector.
    thickness : scalar
        The tickness of the ring, represented as a fraction of the radius.
    N : int
        The number of subdivisions around its axis. If smaller
        than 8, flat shading is used instead of smooth shading.
    M : int
        The number of subdivisions along its axis. If smaller
        than 8, flat shading is used instead of smooth shading.
    axesAdjust : bool
        If True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet,
        it is set to False.
    axes : Axes instance
        Display the bars in the given axes, or the current axes if not given.
    
    """
    
    # Note that the number of vertices around the axis is N+1. This
    # would not be necessary per see, but it helps create a nice closed
    # texture when it is mapped. There are N number of faces though.
    # Similarly, to obtain M faces along the axis, we need M+1
    # vertices.
    
    # Quick access
    pi2 = np.pi*2
    cos = np.cos
    sin = np.sin
    sl = M+1
    
    # Determine where the stitch is, depending on M
    if M<=8:
        rotOffset = 0.5/M
    else:
        rotOffset = 0.0
    
    # Calculate vertices, normals and texcords
    vertices = Pointset(3)
    normals = Pointset(3)
    texcords = Pointset(2)
    # Cone
    for n in range(N+1):
        v = float(n)/N
        a = pi2 * v
        # Obtain outer and center position of "tube"
        po = Point(sin(a), cos(a), 0)
        pc = po * (1.0-0.5*thickness)
        # Create two vectors that span the the circle orthogonal to the tube
        p1 = (pc-po)
        p2 = Point(0, 0, 0.5*thickness)
        # Sample around tube
        for m in range(M+1):
            u = float(m) / (M)
            b = pi2 * (u+rotOffset)
            dp = cos(b) * p1 + sin(b) * p2
            vertices.append(pc+dp)
            normals.append(dp.normalize())
            texcords.append(v,u)
    
    # Calculate indices
    indices = []
    for j in range(N):
        for i in range(M):
            #indices.extend([j*sl+i, j*sl+i+1, (j+1)*sl+i+1, (j+1)*sl+i])
            indices.extend([(j+1)*sl+i, (j+1)*sl+i+1, j*sl+i+1, j*sl+i])
    
    # Make indices a numpy array
    indices = np.array(indices, dtype=np.uint32)
    
    
    ## Visualize
    
    # Create axes
    if axes is None:
        axes = vv.gca()
    
    # Create mesh
    m = vv.OrientableMesh(axes, vertices, indices, normals, values=texcords,
            verticesPerFace=4)
    #
    if translation is not None:
        m.translation = translation
    if scaling is not None:
        m.scaling = scaling
    if direction is not None:
        m.direction = direction
    if rotation is not None:
        m.rotation = rotation
    
    # If necessary, use flat shading
    if N<8 or M<8:
        m.faceShading = 'flat'
    
    # Adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = False
        axes.cameraType = '3d'
        axes.SetLimits()
    
    # Done
    axes.Draw()
    return m


if __name__ == '__main__':
    m1 = vv.solidRing((0,0,1), N=64, M=4, thickness=0.5)
    m2 = vv.solidRing((0,0,2), N=64, thickness=0.25)
    m1.SetTexture( vv.imread('astronaut.png') )



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def draw(figure=None, fast=False):
    """ draw(figure=None, fast=False)
    
    Makes the given figure (or the current figure if None) draw itself.
    If fast is True, some wobjects can draw itself faster at reduced
    quality.
    
    This function is now more or less deprecated; visvis is designed to
    invoke a draw whenever necessary.
    
    """
    
    # Get figure
    if figure is None:
        figure = vv.gcf()
    
    # Draw!
    figure.Draw(fast)


if __name__ == '__main__':
    import time
    fig = vv.figure()
    l = vv.plot([1,2,3,1,4])
    for i in range(20):
        l.SetYdata([1+i/10.0, 2,3,1,4])
        vv.draw(fig) # Note: not even necessary
        vv.processEvents()
        time.sleep(0.1)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import numpy as np

def unwindFaces(mesh):
    """ unwindFaces(mesh)
    
    Unwinds the faces to make new versions of the vertices, normals,
    values, which are usually larger. The new arrays
    represent the same surface, but is described without a faces
    array.
    
    """
    
    # Make new vertices and normals if faces are used
    if mesh._faces is None:
        return
    
    # Get references and new size
    faces = mesh._faces
    N = faces.shape[0]
    
    # Unwind vertices
    if mesh._vertices is not None:
        # Get ref and allocate new array
        vertices = mesh._vertices
        newVertices = np.zeros((N,3), dtype='float32')
        # Unwind
        for i in range(N):
            newVertices[i,:] = vertices[faces[i]]
        # Store
        mesh._vertices = newVertices
    
    # Unwind normals
    if mesh._normals is not None:
        # Get ref and allocate new array
        normals = mesh._normals
        newNormals = np.zeros((N,3), dtype='float32')
        for i in range(N):
            newNormals[i,:] = normals[faces[i]]
        # Store
        mesh._normals = newNormals
        mesh._flatNormals = None
    
    # Unwind values
    if mesh._values is not None:
        # Get ref and allocate new array
        values = mesh._values
        M = values.shape[1]
        newValues = np.zeros((N,M), dtype='float32')
        for i in range(N):
            newValues[i,:] = values[faces[i]]
        # Store
        mesh._values = newValues
    
    # Remove reference to faces
    mesh._faces = None
